@@ -174,6 +174,8 @@ if [ "${DO_CLEAN}" == "1" ]; then
 		tail -n 20 "${CLEAN_LOG}"
 		exit 1
 	fi
+	#clean comtypes cache - it should not be included!
+	python $MINGW_PREFIX/bin/clear_comtypes_cache.py -y
 fi
 
 if [ "${DO_SERVICE}" == "1" ]; then
@@ -264,20 +266,24 @@ if [ "${DO_SERVICE}" == "1" ]; then
 fi
 
 if [ "${DO_CUDA}" == "1" ]; then
-	echo "* Building CUDA kernels"
-	cmd.exe //c "packaging\\MSWindows\\BUILD_CUDA_KERNEL" BGRX_to_NV12 || exit 1
-	cmd.exe //c "packaging\\MSWindows\\BUILD_CUDA_KERNEL" BGRX_to_YUV444 || exit 1
-	cmd.exe //c "packaging\\MSWindows\\BUILD_CUDA_KERNEL" XRGB_to_NV12 || exit 1
-	cmd.exe //c "packaging\\MSWindows\\BUILD_CUDA_KERNEL" XRGB_to_YUV444 || exit 1
-	cmd.exe //c "packaging\\MSWindows\\BUILD_CUDA_KERNEL" BGRX_to_RGB || exit 1
-	cmd.exe //c "packaging\\MSWindows\\BUILD_CUDA_KERNEL" RGBX_to_RGB || exit 1
+	echo "* Building all the CUDA kernels"
+	BUILD_CUDA_KERNEL="packaging\\MSWindows\\BUILD_CUDA_KERNEL"
+	for cupath in `ls fs/share/xpra/cuda/*.cu`; do
+		cu=`basename $cupath`
+		fatbin="${cupath/.cu/.fatbin}"
+		if [[ "$cupath" -nt "$fatbin" ]] || [[ "$BUILD_CUDA_KERNEL" -nt "fatbin" ]]; then
+			kname="${cu%.*}"
+			cmd.exe //c "${BUILD_CUDA_KERNEL}" "$kname" || exit 1
+		fi
+	done
 else
 	BUILD_OPTIONS="${BUILD_OPTIONS} --without-nvenc"
 fi
 
 echo "* Building Python3 Cython modules"
 BUILD_LOG="packaging/MSWindows/Python3-build.log"
-${PYTHON} ./setup.py build_ext ${BUILD_OPTIONS} --inplace -j `nproc` >& ${BUILD_LOG}
+NPROCS=${NPROCS:-`nproc`}
+${PYTHON} ./setup.py build_ext ${BUILD_OPTIONS} --inplace -j ${NPROCS} >& ${BUILD_LOG}
 if [ "$?" != "0" ]; then
 	echo "ERROR: build failed, see ${BUILD_LOG}:"
 	tail -n 20 "${BUILD_LOG}"
@@ -551,7 +557,7 @@ if [ "${DO_INSTALLER}" == "1" ]; then
 	if [ "${DO_SIGN}" == "1" ]; then
 		SIGNTOOL_LOG="packaging/MSWindows/signtool.log"
 		echo "* Signing EXE"
-		cmd.exe //c signtool.exe sign //v //f "${KEY_FILE}" //t "http://timestamp.comodoca.com/authenticode" "${INSTALLER_FILENAME}" > ${SIGNTOOL_LOG}
+		cmd.exe //c signtool.exe sign //v //f "${KEY_FILE}" //t "http://timestamp.digicert.com" "${INSTALLER_FILENAME}" > ${SIGNTOOL_LOG}
 		if [ "$?" != 0 ]; then
 			echo "signtool command failed, see ${SIGNTOOL_LOG}:"
 			cat ${SIGNTOOL_LOG}
