@@ -22,7 +22,7 @@ if [ "${CLIENT_ONLY}" == "1" ]; then
 	DO_TESTS="0"
 else
 	if [ ! -e "${JHBUILD_PREFIX}/share/xpra/www/" ]; then
-		echo "the xpra html5 client must be installed in ${JHBUILD_PREFIX}/share/xpra/www/" 
+		echo "the xpra html5 client must be installed in ${JHBUILD_PREFIX}/share/xpra/www/"
 		exit 1
 	fi
 fi
@@ -72,7 +72,7 @@ fi
 INSTALL_LOG=`pwd`/install.log
 echo "./setup.py install ${BUILD_ARGS}"
 echo " (see ${INSTALL_LOG} for details)"
-${PYTHON} ./setup.py install ${BUILD_ARGS} >& ${INSTALL_LOG}
+${PYTHON} ./setup.py install --old-and-unmanageable ${BUILD_ARGS} >& ${INSTALL_LOG}
 if [ "$?" != "0" ]; then
 	popd
 	echo "ERROR: install failed"
@@ -340,7 +340,7 @@ ln -sf Resources/lib Frameworks
 pushd Resources/lib
 if [ "$STRIP_GSTREAMER_PLUGINS" == "1" ]; then
 	echo "removing extra gstreamer dylib deps:"
-	for x in check photography rtsp sdp; do
+	for x in check photography; do
 		echo "* removing "$x
 		rm libgst${x}*
 	done
