@@ -1,6 +1,6 @@
-#!/usr/bin/env python33
+#!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2018, 2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2018-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -9,8 +9,8 @@ import os.path
 from io import BytesIO
 from math import cos, sin
 
-from xpra.util import typedict, envint, AdHocStruct, AtomicInteger, iround
-from xpra.os_util import WIN32
+from xpra.util import typedict, envint, AdHocStruct, AtomicInteger
+from xpra.os_util import WIN32, load_binary_file
 from xpra.log import Logger
 from xpra.platform.paths import get_icon_filename
 
@@ -32,8 +32,6 @@ def get_gl_client_window_module(force_enable=False):
         return opengl_props, nativegl_client_window
     return {}, None
 
-def noop(*_args):
-    pass
 def no_scaling(*args):
     if len(args)==1:
         return args[0]
@@ -59,6 +57,11 @@ class FakeClient(AdHocStruct):
         self.encoding_defaults = {}
         self.get_window_frame_sizes = get_None
         self._focused = None
+        self.wheel_smooth = False
+        self.pointer_grabbed = None
+        def noop(*_args):
+            """ pretend this method exists and does something """
+        self.find_window = noop
         self.request_frame_extents = noop
         self.server_window_states = ()
         self.server_window_frame_extents = False
@@ -99,12 +102,12 @@ def test_gl_client_window(gl_client_window_class, max_window_size=(1024, 1024),
         noclient = FakeClient()
         #test with alpha, but not on win32
         #because we can't do alpha on win32 with opengl
-        metadata = typedict({b"has-alpha" : not WIN32})
+        metadata = typedict({"has-alpha" : not WIN32})
         class NoHeaderGLClientWindow(gl_client_window_class):
             def add_header_bar(self):
-                pass
+                """ pretend to add the header bar """
             def schedule_recheck_focus(self):
-                pass
+                """ pretend to handle focus checks """
         window = NoHeaderGLClientWindow(noclient, None, None, 2**32-1, x, y, w, h, w, h,
                                         metadata, False, typedict({}),
                                         border, max_window_size, default_cursor_data, pixel_depth)
@@ -132,7 +135,7 @@ def test_gl_client_window(gl_client_window_class, max_window_size=(1024, 1024),
         gl_icon = get_icon_filename("opengl", ext="png")
         icon_data = None
         if os.path.exists(gl_icon):
-            from PIL import Image
+            from PIL import Image  # @UnresolvedImport
             img = Image.open(gl_icon)
             img.load()
             icon_w, icon_h = img.size
@@ -140,10 +143,16 @@ def test_gl_client_window(gl_client_window_class, max_window_size=(1024, 1024),
             noalpha = Image.new("RGB", img.size, (255, 255, 255))
             noalpha.paste(img, mask=img.split()[3]) # 3 is the alpha channel
             buf = BytesIO()
-            noalpha.save(buf, format="JPEG")
-            icon_data = buf.getvalue()
-            buf.close()
-            icon_format = "jpeg"
+            try:
+                noalpha.save(buf, format="JPEG")
+                icon_data = buf.getvalue()
+                buf.close()
+                icon_format = "jpeg"
+            except KeyError as e:
+                log("save()", exc_info=True)
+                log.warn("OpenGL using png as jpeg is not supported by Pillow: %s", e)
+                icon_data = load_binary_file(gl_icon)
+                icon_format = "png"
         if not icon_data:
             icon_w = 32
             icon_h = 32
@@ -158,8 +167,8 @@ def test_gl_client_window(gl_client_window_class, max_window_size=(1024, 1024),
             options["flush"] = 0
             mx = w//2-icon_w//2
             my = h//2-icon_h//2
-            x = iround(mx*(1+sin(v/100)))
-            y = iround(my*(1+cos(v/100)))
+            x = round(mx*(1+sin(v/100)))
+            y = round(my*(1+cos(v/100)))
             window.draw_region(x, y, icon_w, icon_h, icon_format, icon_data, icon_stride, v, options, [paint_callback])
             return REPAINT_DELAY>0
         #the paint code is actually synchronous here,
@@ -175,6 +184,10 @@ def test_gl_client_window(gl_client_window_class, max_window_size=(1024, 1024),
             Gtk.main()
         else:
             draw()
+            #ugly workaround for calling the paint handler
+            #when the main loop is not running:
+            if hasattr(window_backing, "on_realize"):
+                window_backing.on_realize()
         if window_backing.last_present_fbo_error:
             return {
                 "success" : False,
@@ -184,7 +197,7 @@ def test_gl_client_window(gl_client_window_class, max_window_size=(1024, 1024),
         if window:
             window.destroy()
     log("test_gl_client_window(..) draw_result=%s", draw_result)
-    return draw_result
+    return draw_result or {"success" : False, "message" : "not painted on screen"}
 
 
 
