@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2022 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -9,7 +9,8 @@ import os
 import weakref
 from time import monotonic
 from subprocess import Popen, PIPE
-from gi.repository import Gtk, Gdk, GdkPixbuf
+from threading import Event
+from gi.repository import Gtk, Gdk, GdkPixbuf  # @UnresolvedImport
 
 from xpra.client.gtk_base.gtk_client_window_base import HAS_X11_BINDINGS, XSHAPE
 from xpra.util import (
@@ -18,29 +19,29 @@ from xpra.util import (
     DEFAULT_METADATA_SUPPORTED, XPRA_OPENGL_NOTIFICATION_ID,
     )
 from xpra.os_util import (
-    bytestostr, strtobytes, hexstr, load_binary_file,
+    bytestostr, strtobytes, hexstr, load_binary_file, osexpand,
     WIN32, OSX, POSIX, is_Wayland,
     )
+from xpra.common import FULL_INFO
 from xpra.simple_stats import std_unit
-from xpra.exit_codes import EXIT_PASSWORD_REQUIRED
-from xpra.scripts.config import TRUE_OPTIONS, FALSE_OPTIONS
-from xpra.make_thread import start_thread
+from xpra.scripts.config import TRUE_OPTIONS, FALSE_OPTIONS, InitExit
 from xpra.gtk_common.cursor_names import cursor_types
 from xpra.gtk_common.gtk_util import (
     get_gtk_version_info, scaled_image, get_default_cursor, color_parse,
     get_icon_pixbuf,
     get_pixbuf_from_data,
     get_default_root_window, get_root_size,
-    get_screen_sizes, GDKWindow,
+    get_screen_sizes, get_monitors_info,
+    GDKWindow,
     GRAB_STATUS_STRING,
     )
+from xpra.exit_codes import EXIT_PASSWORD_REQUIRED
 from xpra.gtk_common.gobject_util import no_arg_signal
 from xpra.client.ui_client_base import UIXpraClient
 from xpra.client.gobject_client_base import GObjectXpraClient
 from xpra.client.gtk_base.gtk_keyboard_helper import GTKKeyboardHelper
 from xpra.client.gtk_base.css_overrides import inject_css_overrides
 from xpra.client.mixins.window_manager import WindowClient
-from xpra.platform.paths import get_icon_filename
 from xpra.platform.gui import force_focus
 from xpra.platform.gui import (
     get_window_frame_sizes, get_window_frame_size,
@@ -143,22 +144,22 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             send_wm_request_frame_extents(root, win)
 
     def run(self):
-        log("run() HAS_X11_BINDINGS=%s", HAS_X11_BINDINGS)
+        log(f"run() HAS_X11_BINDINGS={HAS_X11_BINDINGS}")
         if HAS_X11_BINDINGS:
             self.setup_frame_request_windows()
         UIXpraClient.run(self)
         self.gtk_main()
-        log("GTKXpraClient.run_main_loop() main loop ended, returning exit_code=%s", self.exit_code)
+        log(f"GTKXpraClient.run_main_loop() main loop ended, returning exit_code={self.exit_code}", )
         return self.exit_code
 
     def gtk_main(self):
-        log("GTKXpraClient.gtk_main() calling %s", Gtk.main)
+        log(f"GTKXpraClient.gtk_main() calling {Gtk.main}",)
         Gtk.main()
         log("GTKXpraClient.gtk_main() ended")
 
 
     def quit(self, exit_code=0):
-        log("GTKXpraClient.quit(%s) current exit_code=%s", exit_code, self.exit_code)
+        log(f"GTKXpraClient.quit({exit_code}) current exit_code={self.exit_code}")
         if self.exit_code is None:
             self.exit_code = exit_code
         if Gtk.main_level()>0:
@@ -167,21 +168,19 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             #try harder!:
             self.timeout_add(5*1000, self.force_quit)
         self.cleanup()
-        log("GTKXpraClient.quit(%s) cleanup done, main_level=%s",
-            exit_code, Gtk.main_level())
+        log(f"GTKXpraClient.quit({exit_code}) cleanup done, main_level={Gtk.main_level()}")
         if Gtk.main_level()>0:
-            log("GTKXpraClient.quit(%s) main loop at level %s, calling gtk quit via timeout",
-                exit_code, Gtk.main_level())
+            log(f"GTKXpraClient.quit({exit_code}) main loop at level {Gtk.main_level()}, calling gtk quit via timeout")
             self.timeout_add(500, self.exit)
 
     def force_quit(self):
         from xpra.os_util import force_quit
-        log("GTKXpraClient.force_quit() calling %s", force_quit)
+        log(f"GTKXpraClient.force_quit() calling {force_quit}")
         force_quit()
 
     def exit(self):
         self.show_progress(100, "terminating")
-        log("GTKXpraClient.exit() calling %s", Gtk.main_quit)
+        log(f"GTKXpraClient.exit() calling {Gtk.main_quit}", )
         Gtk.main_quit()
 
     def cleanup(self):
@@ -231,12 +230,15 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
 
 
     def get_vrefresh(self):
+        rate = envint("XPRA_VREFRESH", 0)
+        if rate:
+            return rate
         #try via GTK:
         rates = {}
         display = Gdk.Display.get_default()
         for m in range(display.get_n_monitors()):
             monitor = display.get_monitor(m)
-            log("monitor %i (%s) refresh-rate=%i", m, monitor.get_model(), monitor.get_refresh_rate())
+            log(f"monitor {m} ({monitor.get_model()}) refresh-rate={monitor.get_refresh_rate()}")
             rates[m] = monitor.get_refresh_rate()
         rate = -1
         if rates:
@@ -261,7 +263,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         except Exception as e:
             notifylog("get_notifier_classes()", exc_info=True)
             notifylog.warn("Warning: cannot load GTK notifier:")
-            notifylog.warn(" %s", e)
+            notifylog.warn(f" {e}")
         return ncs
 
 
@@ -270,19 +272,16 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         Gdk.notify_startup_complete()
 
 
-    def do_process_challenge_prompt(self, packet, prompt="password"):
+    def do_process_challenge_prompt(self, prompt="password"):
         authlog = Logger("auth")
         self.show_progress(100, "authentication")
         PINENTRY = os.environ.get("XPRA_PINENTRY", "")
         from xpra.scripts.pinentry_wrapper import get_pinentry_command
         pinentry_cmd = get_pinentry_command(PINENTRY)
-        authlog("do_process_challenge_prompt%s get_pinentry_command(%s)=%s",
-                (packet, prompt), PINENTRY, pinentry_cmd)
+        authlog(f"do_process_challenge_prompt({prompt}) get_pinentry_command({PINENTRY})={pinentry_cmd}")
         if pinentry_cmd:
-            start_thread(self.handle_challenge_with_pinentry,
-                         "pinentry", True, (packet, prompt, pinentry_cmd))
-            return True
-        return self.do_process_challenge_prompt_dialog(packet, prompt)
+            return self.handle_challenge_with_pinentry(prompt, pinentry_cmd)
+        return self.process_challenge_prompt_dialog(prompt)
 
     def stop_pinentry(self):
         pp = self.pinentry_proc
@@ -292,44 +291,63 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
 
     def get_server_authentication_string(self):
         p = self._protocol
+        server_type = ""
         if p:
             server_type = {
                 "xpra"  : "Xpra ",
                 "rfb"   : "VNC ",
                 }.get(p.TYPE, p.TYPE)
-        else:
-            server_type = ""
-        return "%sServer Authentication:" % server_type
+        return f"{server_type}Server Authentication:"
 
-    def handle_challenge_with_pinentry(self, packet, prompt="password", cmd="pinentry"):
+    def handle_challenge_with_pinentry(self, prompt="password", cmd="pinentry"):
+        # pylint: disable=import-outside-toplevel
         authlog = Logger("auth")
-        authlog("handle_challenge_with_pinentry%s", (packet, prompt, cmd))
+        authlog("handle_challenge_with_pinentry%s", (prompt, cmd))
         try:
             proc = Popen([cmd], stdin=PIPE, stdout=PIPE, stderr=PIPE)
         except OSError:
             authlog("pinentry failed", exc_info=True)
-        else:
-            self.pinentry_proc = proc
-            q = "Enter %s" % prompt
-            p = self._protocol
-            if p:
-                conn = getattr(p, "_conn", None)
-                if conn:
-                    from xpra.net.bytestreams import pretty_socket
-                    cinfo = conn.get_info()
-                    endpoint = pretty_socket(cinfo.get("endpoint", conn.target)).split("?")[0]
-                    q += "\n at %s" % endpoint
-            def got_pin(value):
-                self.idle_add(self.send_challenge_reply, packet, value)
-            def no_pin():
-                self.idle_add(self.quit, EXIT_PASSWORD_REQUIRED)
-            from xpra.scripts.pinentry_wrapper import pinentry_getpin
-            title = self.get_server_authentication_string()
-            pinentry_getpin(proc, title, q, got_pin, no_pin)
-            return True
-        return self.do_process_challenge_prompt_dialog(packet, prompt)
+            return self.process_challenge_prompt_dialog(prompt)
+        self.pinentry_proc = proc
+        q = f"Enter {prompt}"
+        p = self._protocol
+        if p:
+            conn = getattr(p, "_conn", None)
+            if conn:
+                from xpra.net.bytestreams import pretty_socket
+                cinfo = conn.get_info()
+                endpoint = pretty_socket(cinfo.get("endpoint", conn.target)).split("?")[0]
+                q += f"\n at {endpoint}"
+        title = self.get_server_authentication_string()
+        values = []
+        errs = []
+        def rec(value=None):
+            values.append(value)
+        def err(value=None):
+            errs.append(value)
+        from xpra.scripts.pinentry_wrapper import pinentry_getpin
+        pinentry_getpin(proc, title, q, rec, err)
+        if not values:
+            if errs and errs[0].startswith("ERR 83886179"):
+                #ie 'ERR 83886179 Operation cancelled <Pinentry>'
+                raise InitExit(EXIT_PASSWORD_REQUIRED, errs[0][len("ERR 83886179"):])
+            return None
+        return values[0]
 
-    def do_process_challenge_prompt_dialog(self, packet, prompt="password"):
+    def process_challenge_prompt_dialog(self, prompt="password"):
+        #challenge handlers run in a separate 'challenge' thread
+        #but we need to run in the UI thread to access the GUI with Gtk
+        #so we block the current thread using an event:
+        wait = Event()
+        values = []
+        self.idle_add(self.do_process_challenge_prompt_dialog, values, wait, prompt)
+        wait.wait()
+        if not values:
+            return None
+        return values[0]
+
+    def do_process_challenge_prompt_dialog(self, values : list, wait : Event, prompt="password"):
+        # pylint: disable=import-outside-toplevel
         title = self.get_server_authentication_string()
         dialog = Gtk.Dialog(title,
                None,
@@ -343,8 +361,8 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             a.set_padding(padding, padding, padding, padding)
             dialog.vbox.pack_start(a)
         import gi
-        gi.require_version("Pango", "1.0")
-        from gi.repository import Pango
+        gi.require_version("Pango", "1.0")  # @UndefinedVariable
+        from gi.repository import Pango  # @UnresolvedImport
         title = Gtk.Label(title)
         title.modify_font(Pango.FontDescription("sans 14"))
         add(title, 16)
@@ -363,10 +381,17 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             password = dialog.password_input.get_text()
             dialog.hide()
             dialog.destroy()
+            response_str = dict((getattr(Gtk.ResponseType, k, ""), k) for k in (
+                "ACCEPT", "APPLY", "CANCEL", "CLOSE", "DELETE_EVENT", "HELP", "NO", "NONE", "OK", "REJECT", "YES"))
+            log(f"handle_response({dialog}, {response}) response={response_str.get(response)}")
             if response!=Gtk.ResponseType.ACCEPT or not password:
-                self.quit(EXIT_PASSWORD_REQUIRED)
-                return
-            self.send_challenge_reply(packet, password)
+                values.append(None)
+                #for those responses, we assume that the user wants to abort authentication:
+                if response in (Gtk.ResponseType.CLOSE, Gtk.ResponseType.REJECT, Gtk.ResponseType.DELETE_EVENT):
+                    self.disconnect_and_quit(EXIT_PASSWORD_REQUIRED, "password entry was cancelled")
+            else:
+                values.append(password)
+            wait.set()
         def password_activate(*_args):
             handle_response(dialog, Gtk.ResponseType.ACCEPT)
         password_input.connect("activate", password_activate)
@@ -375,7 +400,6 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             from xpra.platform.darwin.gui import enable_focus_workaround
             enable_focus_workaround()
         dialog.show()
-        return True
 
 
     def setup_connection(self, conn):
@@ -411,13 +435,13 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             if endpoint:
                 m.update(strtobytes(endpoint))
             color_str = "#%s" % m.hexdigest()[:6]
-            log("border color derived from %s: %s", endpoint, color_str)
+            log(f"border color derived from {endpoint}: {color_str}")
         try:
             color = color_parse(color_str)
             assert color is not None
         except Exception as e:
             if warn:
-                log.warn("Warning: invalid border color specified '%s'", color_str)
+                log.warn(f"Warning: invalid border color specified '{color_str!r}'")
                 if str(e):
                     log.warn(" %s", e)
                 self.show_border_help()
@@ -430,14 +454,14 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                 size = int(size_str)
             except Exception as e:
                 if warn:
-                    log.warn("Warning: invalid border size specified '%s'", size_str)
-                    log.warn(" %s", e)
+                    log.warn(f"Warning: invalid border size specified {size_str!r}")
+                    log.warn(f" {e}")
                     self.show_border_help()
             if size<=0:
-                log("border size is %s, disabling it", size)
+                log(f"border size is {size}, disabling it")
                 return
             if size>=45:
-                log.warn("Warning: border size is too large: %s, clipping it", size)
+                log.warn(f"Warning: border size is too large: {size}, clipping it")
                 size = 45
         from xpra.client.window_border import WindowBorder
         self.border = WindowBorder(enabled, color.red/65536.0, color.green/65536.0, color.blue/65536.0, alpha, size)
@@ -459,8 +483,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             log.warn("Warning: cannot start new commands")
             log.warn(" the feature is currently disabled on the server")
             return
-        log("show_start_new_command%s current start_new_command=%s, flag=%s",
-            args, self.start_new_command, self.server_start_new_commands)
+        log(f"show_start_new_command{args} current start_new_command={self.start_new_command}, flag={self.server_start_new_commands}")
         if self.start_new_command is None:
             from xpra.client.gtk_base.start_new_command import getStartNewCommand
             def run_command_cb(command, sharing=True):
@@ -471,6 +494,17 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         self.start_new_command.show()
 
 
+    ################################
+    # monitors
+    def send_remove_monitor(self, index):
+        assert self.server_monitors
+        self.send("configure-monitor", "remove", "index", index)
+
+    def send_add_monitor(self, resolution="1024x768"):
+        assert self.server_monitors
+        self.send("configure-monitor", "add", resolution)
+
+
     ################################
     # file handling
     def ask_data_request(self, cb_answer, send_id, dtype, url, filesize, printit, openit):
@@ -510,7 +544,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         #check if we have accepted this file via the GUI:
         r = self.data_send_requests.pop(send_id, None)
         if not r:
-            filelog("accept_data: data send request %s not found", send_id)
+            filelog(f"accept_data: data send request {send_id} not found")
             from xpra.net.file_transfer import FileTransferHandler
             return FileTransferHandler.accept_data(self, send_id, dtype, url, printit, openit)
         edtype = r[0]
@@ -535,9 +569,9 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             self.file_size_dialog = None
         parent = None
         msgs = (
-                "Warning: cannot %s the file '%s'" % (action, basefilename),
-                "this file is too large: %sB" % std_unit(filesize),
-                "the %s file size limit is %iB" % (location, std_unit(limit)),
+                f"Warning: cannot {action} the file {basefilename!r}",
+                f"this file is too large: {std_unit(filesize)}B",
+                f"the {location} file size limit is {std_unit(limit)}B",
                 )
         self.file_size_dialog = Gtk.MessageDialog(parent, Gtk.DialogFlags.DESTROY_WITH_PARENT, Gtk.MessageType.INFO,
                                                   Gtk.ButtonsType.CLOSE, "\n".join(msgs))
@@ -545,7 +579,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             image = Gtk.Image.new_from_stock(Gtk.STOCK_DIALOG_WARNING, Gtk.IconSize.BUTTON)
             self.file_size_dialog.set_image(image)
         except Exception as e:
-            log.warn("Warning: failed to set dialog image: %s", e)
+            log.warn(f"Warning: failed to set dialog image: {e}")
         self.file_size_dialog.connect("response", self.close_file_size_warning)
         self.file_size_dialog.show()
 
@@ -569,7 +603,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         if self.file_dialog:
             self.file_dialog.present()
             return
-        filelog("show_file_upload%s can open=%s", args, self.remote_open_files)
+        filelog(f"show_file_upload{args} can open={self.remote_open_files}")
         buttons = [Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL]
         if self.remote_open_files:
             buttons += [Gtk.STOCK_OPEN,      Gtk.ResponseType.ACCEPT]
@@ -591,11 +625,11 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
 
     def file_upload_dialog_response(self, dialog, v):
         if v not in (Gtk.ResponseType.OK, Gtk.ResponseType.ACCEPT):
-            filelog("dialog response code %s", v)
+            filelog(f"dialog response code {v}")
             self.close_file_upload_dialog()
             return
         filename = dialog.get_filename()
-        filelog("file_upload_dialog_response: filename=%s", filename)
+        filelog("file_upload_dialog_response: filename={filename!r}")
         try:
             filesize = os.stat(filename).st_size
         except OSError:
@@ -606,7 +640,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                 return
         gfile = dialog.get_file()
         self.close_file_upload_dialog()
-        filelog("load_contents: filename=%s, response=%s", filename, v)
+        filelog(f"load_contents: filename={filename!r}, response={v}")
         cancellable = None
         user_data = (filename, v==Gtk.ResponseType.ACCEPT)
         gfile.load_contents_async(cancellable, self.file_upload_ready, user_data)
@@ -616,12 +650,11 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         filename, openit = user_data
         _, data, entity = gfile.load_contents_finish(result)
         filesize = len(data)
-        filelog("load_contents_finish(%s)=%s", result, (type(data), filesize, entity))
+        filelog(f"load_contents_finish({result})=%s", (type(data), filesize, entity))
         if not data:
-            log.warn("Warning: failed to load file '%s'", filename)
+            log.warn(f"Warning: failed to load file {filename!r}")
             return
-        filelog("load_contents: filename=%s, %i bytes, entity=%s, openit=%s",
-                filename, filesize, entity, openit)
+        filelog(f"load_contents: filename={filename}, {filesize} bytes, entity={entity}, openit={openit}")
         self.send_file(filename, "", data, filesize=filesize, openit=openit)
 
 
@@ -630,6 +663,10 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         force_focus()
         about()
 
+    def show_docs(self, *_args):
+        from xpra.scripts.main import run_docs
+        run_docs()
+
     def show_shortcuts(self, *_args):
         if self.shortcuts_info and not self.shortcuts_info.is_closed:
             force_focus()
@@ -680,7 +717,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         #gives the server time to send an info response..
         #(by the time the user clicks on copy, it should have arrived, we hope!)
         def got_server_log(filename, filesize):
-            log("got_server_log(%s, %s)", filename, filesize)
+            log(f"got_server_log({filename!r}, {filesize})")
             filedata = load_binary_file(filename)
             self.bug_report.set_server_log_data(filedata)
         self.download_server_log(got_server_log)
@@ -690,12 +727,12 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
     def get_image(self, icon_name, size=None):
         try:
             pixbuf = get_icon_pixbuf(icon_name)
-            log("get_image(%s, %s) pixbuf=%s", icon_name, size, pixbuf)
+            log(f"get_image({icon_name!r}, {size}) pixbuf={pixbuf}")
             if not pixbuf:
                 return  None
             return scaled_image(pixbuf, size)
         except Exception:
-            log.error("get_image(%s, %s)", icon_name, size, exc_info=True)
+            log.error(f"Error: get_image({icon_name!r}, {size})", icon_name, size, exc_info=True)
             return None
 
 
@@ -705,7 +742,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         root = self.get_root_window()
         with xsync:
             win = window.get_window()
-            framelog("request_frame_extents(%s) xid=%#x", window, win.get_xid())
+            framelog(f"request_frame_extents({window}) xid={win.get_xid()}")
             send_wm_request_frame_extents(root, win)
 
     def get_frame_extents(self, window):
@@ -713,7 +750,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         x, y = window.get_position()
         w, h = window.get_size()
         v = get_window_frame_size(x, y, w, h)   #pylint: disable=assignment-from-none
-        framelog("get_window_frame_size%s=%s", (x, y, w, h), v)
+        framelog(f"get_window_frame_size{(x, y, w, h)}={v}")
         if v:
             #(OSX does give us these values via Quartz API)
             return v
@@ -724,7 +761,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         gdkwin = window.get_window()
         assert gdkwin
         v = prop_get(gdkwin, "_NET_FRAME_EXTENTS", ["u32"], ignore_errors=False)
-        framelog("get_frame_extents(%s)=%s", window.get_title(), v)
+        framelog(f"get_frame_extents({window.get_title()})={v}")
         return v
 
     def get_window_frame_sizes(self):
@@ -739,19 +776,19 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                 try:
                     if len(v)==8:
                         if first_time("invalid-frame-extents"):
-                            framelog.warn("Warning: invalid frame extents value '%s'", v)
+                            framelog.warn(f"Warning: invalid frame extents value {v!r}")
                             if wm_name:
-                                framelog.warn(" this is probably a bug in '%s'", wm_name)
-                            framelog.warn(" using '%s' instead", v[4:])
+                                framelog.warn(f" this is probably a bug in {wm_name!r}")
+                            framelog.warn(f" using {v[4:]} instead")
                         v = v[4:]
                     l, r, t, b = v
                     wfs["frame"] = (l, r, t, b)
                     wfs["offset"] = (l, t)
                 except Exception as e:
-                    framelog.warn("Warning: invalid frame extents value '%s'", v)
-                    framelog.warn(" %s", e)
-                    framelog.warn(" this is probably a bug in '%s'", wm_name)
-        framelog("get_window_frame_sizes()=%s", wfs)
+                    framelog.warn(f"Warning: invalid frame extents value {v}")
+                    framelog.warn(f" {e}")
+                    framelog.warn(f" this is probably a bug in {wm_name!r}")
+        framelog(f"get_window_frame_sizes()={wfs}")
         return wfs
 
 
@@ -803,7 +840,8 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         capabilities = UIXpraClient.make_hello(self)
         capabilities["named_cursors"] = len(cursor_types)>0
         capabilities["encoding.transparency"] = self.has_transparency()
-        capabilities.update(flatten_dict(get_gtk_version_info()))
+        if FULL_INFO>1:
+            capabilities.update(flatten_dict(get_gtk_version_info()))
         if EXPORT_ICON_DATA:
             #tell the server which icons GTK can use
             #so it knows when it should supply one as fallback
@@ -817,13 +855,13 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                 #if d not in it.get_search_path():
                 #    it.append_search_path(d)
                 #    it.rescan_if_needed()
-                log("default icon theme: %s", it)
-                log("icon search path: %s", it.get_search_path())
-                log("contexts: %s", it.list_contexts())
+                log(f"default icon theme: {it}")
+                log(f"icon search path: {it.get_search_path()}")
+                log(f"contexts: {it.list_contexts()}")
                 icons = []
                 for context in it.list_contexts():
                     icons += it.list_icons(context)
-                log("icons: %s", icons)
+                log(f"icons: {icons}")
                 capabilities["theme.default.icons"] = tuple(set(icons))
         if METADATA_SUPPORTED:
             ms = [x.strip() for x in METADATA_SUPPORTED.split(",")]
@@ -838,8 +876,10 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             ms = list(DEFAULT_METADATA_SUPPORTED)
             #added in 0.15:
             ms += ["command", "workspace", "above", "below", "sticky",
-                   "set-initial-position",  #0.17
+                   "set-initial-position", "requested-position",
                    "content-type",
+                   #4.4:
+                   "parent", "relative-position",
                    ]
         if POSIX:
             #this is only really supported on X11, but posix is easier to check for..
@@ -864,17 +904,23 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                     "icons.size"        : (64, 64),     #size we want
                     "icons.max_size"    : (128, 128),   #limit
                     })
-        capabilities["opengl"] = self.opengl_props
+        if FULL_INFO:
+            capabilities["opengl"] = self.opengl_props
         return capabilities
 
 
     def has_transparency(self) -> bool:
+        if not envbool("XPRA_ALPHA", True):
+            return False
         screen = Gdk.Screen.get_default()
         if screen is None:
             return is_Wayland()
         return screen.get_rgba_visual() is not None
 
 
+    def get_monitors_info(self):
+        return get_monitors_info(self.xscale, self.yscale)
+
     def get_screen_sizes(self, xscale=1, yscale=1):
         return get_screen_sizes(xscale, yscale)
 
@@ -886,12 +932,12 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
 
 
     def set_windows_cursor(self, windows, cursor_data):
-        cursorlog("set_windows_cursor(%s, args[%i])", windows, len(cursor_data))
+        cursorlog(f"set_windows_cursor({windows}, args[{len(cursor_data)}])")
         cursor = None
         if cursor_data:
             try:
                 cursor = self.make_cursor(cursor_data)
-                cursorlog("make_cursor(..)=%s", cursor)
+                cursorlog(f"make_cursor(..)={cursor}")
             except Exception as e:
                 log.warn("error creating cursor: %s (using default)", e, exc_info=True)
             if cursor is None:
@@ -973,7 +1019,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             #OS wants a fixed cursor size! (win32 does, and GTK doesn't do this for us)
             if w<=fw and h<=fh:
                 cursorlog("pasting %ix%i cursor to fixed OS size %ix%i", w, h, fw, fh)
-                from PIL import Image
+                from PIL import Image  # @UnresolvedImport
                 img = Image.frombytes("RGBA", (w, h), pixels, "raw", "BGRA", w*4, 1)
                 target = Image.new("RGBA", (fw, fh))
                 target.paste(img, (0, 0, w, h))
@@ -1068,7 +1114,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             gdkwindow = window.get_window()
         if gdkwindow is None:
             gdkwindow = self.get_root_window()
-        log("window_bell(..) gdkwindow=%s", gdkwindow)
+        log(f"window_bell(..) gdkwindow={gdkwindow}")
         if not system_bell(gdkwindow, device, percent, pitch, duration, bell_class, bell_id, bell_name):
             #fallback to simple beep:
             Gdk.beep()
@@ -1077,7 +1123,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
     def _process_raise_window(self, packet):
         wid = packet[1]
         window = self._id_to_window.get(wid)
-        focuslog("going to raise window %s - %s", wid, window)
+        focuslog(f"going to raise window {wid} - {window}")
         if window:
             if window.has_toplevel_focus():
                 log("window already has top level focus")
@@ -1090,9 +1136,9 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         window = self._id_to_window.get(wid)
         other_window = self._id_to_window.get(other_wid)
         focuslog("restack window %s - %s %s %s",
-            wid, window, ["below", "above"][above], other_window)
+            wid, window, ["above", "below"][above], other_window)
         if window:
-            window.get_window().restack(other_window, above)  #Above=0
+            window.restack(other_window, above)
 
     def opengl_setup_failure(self, summary = "Xpra OpenGL Acceleration Failure", body=""):
         OK = "0"
@@ -1103,7 +1149,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                 from xpra.platform.paths import get_user_conf_dirs
                 dirs = get_user_conf_dirs()
                 for d in dirs:
-                    conf_file = os.path.join(d, "xpra.conf")
+                    conf_file = osexpand(os.path.join(d, "xpra.conf"))
                     try:
                         with open(conf_file, 'ab') as f:
                             f.write(b"\n")
@@ -1111,6 +1157,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                             f.write(b"opengl=nowarn\n")
                         log.info("OpenGL warning will be silenced from now on,")
                         log.info(" '%s' has been updated", conf_file)
+                        break
                     except OSError:
                         log("failed to create / append to config file '%s'", conf_file, exc_info=True)
         def delayed_notify():
@@ -1122,7 +1169,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                 log.info(summary)
                 log.info(body)
                 return
-            actions = (OK, "OK", DISABLE, "Disable this warning")
+            actions = (OK, "OK", DISABLE, "Don't show this warning again")
             self.may_notify(XPRA_OPENGL_NOTIFICATION_ID, summary, body, actions,
                             icon_name="opengl", callback=notify_callback)
         #wait for the main loop to run:
@@ -1130,7 +1177,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
 
     #OpenGL bits:
     def init_opengl(self, enable_opengl):
-        opengllog("init_opengl(%s)", enable_opengl)
+        opengllog(f"init_opengl({enable_opengl})")
         #enable_opengl can be True, False, force, probe-failed, probe-success, or None (auto-detect)
         #ie: "on:native,gtk", "auto", "no"
         #ie: "probe-failed:SIGSEGV"
@@ -1138,7 +1185,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         enable_opengl = (enable_opengl or "")
         parts = enable_opengl.split(":", 1)
         enable_option = parts[0].lower()        #ie: "on"
-        opengllog("init_opengl: enable_option=%s", enable_option)
+        opengllog(f"init_opengl: enable_option={enable_option}")
         if enable_option in ("probe-failed", "probe-error", "probe-crash"):
             msg = enable_option.replace("-", " ")
             if len(parts)>1 and any(len(x) for x in parts[1:]):
@@ -1210,7 +1257,11 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             self.gl_texture_size_limit = self.opengl_props.get("texture-size-limit", 16*1024)
             self.gl_max_viewport_dims = self.opengl_props.get("max-viewport-dims",
                                                               (self.gl_texture_size_limit, self.gl_texture_size_limit))
-            driver_info = self.opengl_props.get("renderer") or self.opengl_props.get("vendor") or "unknown card"
+            renderer = self.opengl_props.get("renderer", "unknown")
+            parts = renderer.split("(")
+            if len(parts)>1 and len(parts[0])>10:
+                renderer = parts[0].strip()
+            driver_info = renderer or self.opengl_props.get("vendor") or "unknown card"
             def disablewarn(msg):
                 opengllog.warn("Warning: OpenGL is disabled:")
                 opengllog.warn(" %s", msg)
@@ -1237,7 +1288,8 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                     l = opengllog.warn
                 l("Warning: OpenGL windows will be clamped to the maximum texture size %ix%i",
                   self.gl_texture_size_limit, self.gl_texture_size_limit)
-                l(" for OpenGL %s renderer '%s'", pver(self.opengl_props.get("opengl", "")), self.opengl_props.get("renderer", "unknown"))
+                glver = pver(self.opengl_props.get("opengl", ""))
+                l(f" for OpenGL {glver} renderer {renderer!r}")
             if self.opengl_enabled and enable_opengl!="probe-success" and not self.opengl_force:
                 draw_result = test_gl_client_window(self.GLClientWindowClass, max_window_size=self.max_window_size, pixel_depth=self.pixel_depth)
                 if not draw_result.get("success", False):
@@ -1245,12 +1297,12 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                     return
                 log("OpenGL test rendering succeeded")
             if self.opengl_enabled:
-                opengllog.info("OpenGL enabled with %s", driver_info)
+                opengllog.info(f"OpenGL enabled on {driver_info!r}")
                 #don't try to handle video dimensions bigger than this:
                 mvs = min(8192, self.gl_texture_size_limit)
                 self.video_max_size = (mvs, mvs)
             elif self.client_supports_opengl:
-                opengllog("OpenGL supported with %s, but not enabled", driver_info)
+                opengllog(f"OpenGL supported on {driver_info!r}, but not enabled")
             self.opengl_props["enabled"] = self.opengl_enabled
             if self.opengl_enabled and not warnings and OSX:
                 #non-opengl is slow on MacOS:
@@ -1320,29 +1372,33 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         self.reinit_window_icons()
 
 
+    def find_window(self, metadata, metadata_key="transient-for"):
+        fwid = metadata.intget(metadata_key, -1)
+        log("find_window(%s, %s) wid=%s", metadata, metadata_key, fwid)
+        if fwid>0:
+            return self._id_to_window.get(fwid)
+        return None
+
+    def find_gdk_window(self, metadata, metadata_key="transient-for"):
+        client_window = self.find_window(metadata, metadata_key)
+        if client_window:
+            gdk_window = client_window.get_window()
+            if gdk_window:
+                return gdk_window
+        return None
+
     def get_group_leader(self, wid, metadata, _override_redirect):
-        transient_for = metadata.intget("transient-for", -1)
-        log("get_group_leader: transient_for=%s", transient_for)
-        if transient_for>0:
-            client_window = self._id_to_window.get(transient_for)
-            if client_window:
-                gdk_window = client_window.get_window()
-                if gdk_window:
-                    return gdk_window
+        def find_gdk_window(metadata_key="transient-for"):
+            return self.find_gdk_window(metadata, metadata_key)
+        win = find_gdk_window("group-leader-wid") or find_gdk_window("transient-for") or find_gdk_window("parent")
+        log(f"get_group_leader(..)={win}")
+        if win:
+            return win
         pid = metadata.intget("pid", -1)
         leader_xid = metadata.intget("group-leader-xid", -1)
-        leader_wid = metadata.intget("group-leader-wid", -1)
-        group_leader_window = self._id_to_window.get(leader_wid)
-        if group_leader_window:
-            #leader is another managed window
-            log("found group leader window %s for wid=%s", group_leader_window, leader_wid)
-            return group_leader_window
-        log("get_group_leader: leader pid=%s, xid=%s, wid=%s", pid, leader_xid, leader_wid)
+        log(f"get_group_leader: leader pid={pid}, xid={leader_xid}")
         reftype = "xid"
         ref = leader_xid
-        if ref<0:
-            reftype = "leader-wid"
-            ref = leader_wid
         if ref<0:
             ci = metadata.strtupleget("class-instance")
             if ci:
@@ -1351,11 +1407,6 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             elif pid>0:
                 reftype = "pid"
                 ref = pid
-            elif transient_for>0:
-                #this should have matched a client window above..
-                #but try to use it anyway:
-                reftype = "transient-for"
-                ref = transient_for
             else:
                 #no reference to use
                 return None
@@ -1434,12 +1485,12 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         tray = self.tray
         if not tray or not CLIPBOARD_NOTIFY:
             return
-        clipboardlog("clipboard_notify(%s) notification timer=%s", n, self.clipboard_notification_timer)
+        clipboardlog(f"clipboard_notify({n}) notification timer={self.clipboard_notification_timer}")
         self.cancel_clipboard_notification_timer()
         if n>0 and self.clipboard_enabled:
             self.last_clipboard_notification = monotonic()
             tray.set_icon("clipboard")
-            tray.set_tooltip("%s clipboard requests in progress" % n)
+            tray.set_tooltip(f"{n} clipboard requests in progress")
             tray.set_blinking(True)
         else:
             #no more pending clipboard transfers,
