@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2017-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2017-2022 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -10,10 +10,10 @@ import socket
 import subprocess
 
 import gi
-gi.require_version("Gtk", "3.0")
-gi.require_version("Pango", "1.0")
-gi.require_version("GdkPixbuf", "2.0")
-from gi.repository import Pango, GLib, Gtk, Gio
+gi.require_version("Gtk", "3.0")  # @UndefinedVariable
+gi.require_version("Pango", "1.0")  # @UndefinedVariable
+gi.require_version("GdkPixbuf", "2.0")  # @UndefinedVariable
+from gi.repository import Pango, GLib, Gtk, Gio  # @UnresolvedImport
 
 from xpra.platform.paths import get_xpra_command, get_nodock_command
 from xpra.platform.dotxpra import DotXpra
@@ -36,7 +36,7 @@ log = Logger("client", "util")
 class SessionsGUI(Gtk.Window):
 
     def __init__(self, options, title="Xpra Session Browser"):
-        Gtk.Window.__init__(self)
+        super().__init__()
         self.exit_code = 0
         self.set_title(title)
         self.set_border_width(20)
@@ -115,7 +115,7 @@ class SessionsGUI(Gtk.Window):
         self.vbox.show()
         self.show()
 
-    def show(self):
+    def show(self):  #pylint: disable=arguments-differ
         super().show()
         def show():
             force_focus()
@@ -168,6 +168,9 @@ class SessionsGUI(Gtk.Window):
                     #try to query it
                     try:
                         info = self.get_session_info(sockpath)
+                        if not info:
+                            log(" no data for '%s'", sockpath)
+                            continue
                         if info.get("session-type")=="client":
                             log(" skipped client socket '%s': %s", sockpath, info)
                             continue
@@ -272,7 +275,7 @@ class SessionsGUI(Gtk.Window):
             if uuid:
                 key = uuid
             else:
-                key = (host, display)
+                key = (host.rstrip("."), display)
             log("populate_table: key[%i]=%s", i, key)
             d.setdefault(key, []).append((interface, protocol, name, stype, domain, host, address, port, text))
             #older servers expose the "session-name" as "session":
