@@ -1,11 +1,16 @@
 # This file is part of Xpra.
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2012-2022 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import os
 import hashlib
+from time import monotonic
 from threading import Lock
+from collections import deque
+from PIL import Image, ImageFont, ImageDraw  #@UnresolvedImport
+from gi.repository import GLib
 
 from xpra.net.mmap_pipe import mmap_read
 from xpra.net import compression
@@ -36,6 +41,26 @@ PAINT_BOX = envint("XPRA_PAINT_BOX", 0) or envint("XPRA_OPENGL_PAINT_BOX", 0)
 WEBP_PILLOW = envbool("XPRA_WEBP_PILLOW", False)
 SCROLL_ENCODING = envbool("XPRA_SCROLL_ENCODING", True)
 REPAINT_ALL = envbool("XPRA_REPAINT_ALL", False)
+SHOW_FPS = envbool("XPRA_SHOW_FPS", False)
+
+
+_PIL_font = None
+def load_PIL_font():
+    global _PIL_font
+    if _PIL_font:
+        return _PIL_font
+    for font_file in (
+        "/usr/share/fonts/gnu-free/FreeMono.ttf",
+        "/usr/share/fonts/liberation-mono/LiberationMono-Regular.ttf",
+        ):
+        if os.path.exists(font_file):
+            try:
+                _PIL_font = ImageFont.load_path(font_file)
+                return _PIL_font
+            except OSError:
+                pass
+    _PIL_font = ImageFont.load_default()
+    return _PIL_font
 
 
 #ie:
@@ -80,15 +105,11 @@ def fire_paint_callbacks(callbacks, success=True, message=""):
 def verify_checksum(img_data, options):
     l = options.intget("z.len")
     if l:
-        assert l==len(img_data), "compressed pixel data failed length integrity check: expected %i bytes but got %i" % (l, len(img_data))
-    try:
-        chksum = options.get("z.md5")
-        if chksum:
-            h = hashlib.md5(img_data)
-    except ValueError:
-        chksum = options.get("z.sha1")
-        if chksum:
-            h = hashlib.sha1(img_data)
+        assert l==len(img_data), "compressed pixel data failed length integrity check: expected %i bytes but got %i" % (
+            l, len(img_data))
+    chksum = options.get("z.sha256")
+    if chksum:
+        h = hashlib.sha256(img_data)
     if h:
         hd = h.hexdigest()
         assert chksum==hd, "pixel data failed compressed chksum integrity check: expected %s but got %s" % (chksum, hd)
@@ -130,31 +151,112 @@ class WindowBackingBase:
             self._PIL_encodings = self.pil_decoder.get_encodings()
         self.jpeg_decoder = get_codec("dec_jpeg")
         self.webp_decoder = get_codec("dec_webp")
+        self.spng_decoder = get_codec("dec_spng")
+        self.avif_decoder = get_codec("dec_avif")
+        self.nvjpeg_decoder = get_codec("dec_nvjpeg")
+        self.cuda_context = None
         self.draw_needs_refresh = True
         self.repaint_all = REPAINT_ALL
         self.mmap = None
         self.mmap_enabled = False
+        self.fps_events = deque(maxlen=120)
+        self.fps_buffer_size = 0, 0
+        self.fps_buffer_update_time = 0
+        self.fps_value = 0
+        self.fps_refresh_timer = 0
 
     def idle_add(self, *_args, **_kwargs):
         raise NotImplementedError()
 
+    def get_rgb_formats(self):
+        if self._alpha_enabled:
+            return self.RGB_MODES
+        #remove modes with alpha:
+        return list(filter(lambda mode : mode.find("A")<0,
+                            ["BGRA", "BGRX", "RGBA", "RGBX", "BGR", "RGB", "r210", "BGR565"]))
+
+
     def get_info(self):
         info = {
-            "rgb-formats"   : self.RGB_MODES,
+            "rgb-formats"   : self.get_rgb_formats(),
             "transparency"  : self._alpha_enabled,
             "mmap"          : bool(self.mmap_enabled),
             "size"          : self.size,
             "render-size"   : self.render_size,
             "offsets"       : self.offsets,
+            "fps"           : self.fps_value,
             }
         vd = self._video_decoder
         if vd:
-            info["video-decoder"] = self._video_decoder.get_info()
+            info["video-decoder"] = vd.get_info()
         csc = self._csc_decoder
         if csc:
-            info["csc"] = self._csc_decoder
+            info["csc"] = csc.get_info()
         return info
 
+    def record_fps_event(self):
+        self.fps_events.append(monotonic())
+        now = monotonic()
+        elapsed = now-self.fps_buffer_update_time
+        if elapsed>0.2:
+            self.update_fps()
+
+    def update_fps(self):
+        self.fps_buffer_update_time = monotonic()
+        self.fps_value = self.calculate_fps()
+        if self.is_show_fps():
+            text = "%i fps" % self.fps_value
+            width, height = 64, 32
+            self.fps_buffer_size = (width, height)
+            pixels = self.rgba_text(text, width, height)
+            self.update_fps_buffer(width, height, pixels)
+
+    def update_fps_buffer(self, width, height, pixels):
+        raise NotImplementedError
+
+    def calculate_fps(self):
+        pe = list(self.fps_events)
+        if not pe:
+            return 0
+        e0 = pe[0]
+        now = monotonic()
+        elapsed = now-e0
+        if elapsed<=1 and len(pe)>=5:
+            return len(pe)//elapsed
+        cutoff = now-1
+        count = 0
+        while pe and pe.pop()>=cutoff:
+            count += 1
+        return count
+
+    def is_show_fps(self):
+        if not SHOW_FPS and self.paint_box_line_width<=0:
+            return False
+        #show fps if the value is non-zero:
+        if self.fps_value>0:
+            return True
+        pe = list(self.fps_events)
+        if not pe:
+            return False
+        last_fps_event = pe[-1]
+        #or if there was an event less than N seconds ago:
+        N = 4
+        return monotonic()-last_fps_event<N
+
+    def rgba_text(self, text, width=64, height=32, x=20, y=10, bg=(128, 128, 128, 32)):
+        rgb_format = "RGBA"
+        img = Image.new(rgb_format, (width, height), color=bg)
+        draw = ImageDraw.Draw(img)
+        font = load_PIL_font()
+        draw.text((x, y), text, "blue", font=font)
+        return img.tobytes("raw", rgb_format)
+
+    def cancel_fps_refresh(self):
+        frt = self.fps_refresh_timer
+        if frt:
+            self.fps_refresh_timer = 0
+            GLib.source_remove(frt)
+
 
     def enable_mmap(self, mmap_area):
         self.mmap = mmap_area
@@ -215,7 +317,8 @@ class WindowBackingBase:
             sy, dy = south_y()
         elif g==StaticGravity:
             if first_time("StaticGravity-%i" % self.wid):
-                log.warn("Warning: static gravity is not handled")
+                log.warn("Warning: window %i requested static gravity", self.wid)
+                log.warn(" this is not implemented yet")
         w = min(bw, oldw)
         h = min(bh, oldh)
         return sx, sy, dx, dy, w, h
@@ -276,8 +379,28 @@ class WindowBackingBase:
         #    pass
         return x, y
 
+    def assign_cuda_context(self, opengl=False):
+        if self.cuda_context is None:
+            from xpra.codecs.nvjpeg.decoder import get_default_device  # @NoMove pylint: disable=no-name-in-module, import-outside-toplevel
+            dev = get_default_device()
+            assert dev
+            #make this an opengl compatible context:
+            from xpra.codecs.cuda_common.cuda_context import cuda_device_context
+            self.cuda_context = cuda_device_context(dev.device_id, dev.device, opengl)
+            #create the context now as this is the part that takes time:
+            self.cuda_context.make_context()
+        return self.cuda_context
+
+
+    def free_cuda_context(self):
+        cc = self.cuda_context
+        if cc:
+            self.cuda_context = None
+            cc.free()
 
     def close(self):
+        self.free_cuda_context()
+        self.cancel_fps_refresh()
         self._backing = None
         log("%s.close() video_decoder=%s", self, self._video_decoder)
         #try without blocking, if that fails then
@@ -312,9 +435,9 @@ class WindowBackingBase:
 
     def get_encoding_properties(self):
         return {
-                 "encodings.rgb_formats"    : self.RGB_MODES,
+                 "encodings.rgb_formats"    : self.get_rgb_formats(),
                  "encoding.transparency"    : self._alpha_enabled,
-                 "encoding.full_csc_modes"  : self._get_full_csc_modes(self.RGB_MODES),
+                 "encoding.full_csc_modes"  : self._get_full_csc_modes(self.get_rgb_formats()),
                  "encoding.send-window-size" : True,
                  "encoding.render-size"     : self.render_size,
                  }
@@ -327,7 +450,9 @@ class WindowBackingBase:
         if not self._alpha_enabled:
             target_rgb_modes = tuple(x for x in target_rgb_modes if x.find("A")<0)
         full_csc_modes = getVideoHelper().get_server_full_csc_modes_for_rgb(*target_rgb_modes)
-        full_csc_modes["webp"] = [x for x in rgb_modes if x in ("BGRX", "BGRA", "RGBX", "RGBA")]
+        full_csc_modes["webp"] = tuple(x for x in rgb_modes if x in ("BGRX", "BGRA", "RGBX", "RGBA"))
+        full_csc_modes["jpeg"] = tuple(x for x in rgb_modes if x in ("BGRX", "BGRA", "RGBX", "RGBA", "YUV420P"))
+        full_csc_modes["jpega"] = tuple(x for x in rgb_modes if x in ("BGRA", "RGBA"))
         videolog("_get_full_csc_modes(%s) with target_rgb_modes=%s", rgb_modes, target_rgb_modes)
         for e in sorted(full_csc_modes.keys()):
             modes = full_csc_modes.get(e)
@@ -335,23 +460,40 @@ class WindowBackingBase:
         return full_csc_modes
 
 
-    def unpremultiply(self, img_data):
-        from xpra.codecs.argb.argb import unpremultiply_argb, unpremultiply_argb_in_place   #@UnresolvedImport
-        if not isinstance(img_data, str):
-            try:
-                unpremultiply_argb_in_place(img_data)
-                return img_data
-            except Exception:
-                log.warn("failed to unpremultiply %s (len=%s)" % (type(img_data), len(img_data)))
-        return unpremultiply_argb(img_data)
-
-
     def set_cursor_data(self, cursor_data):
         self.cursor_data = cursor_data
 
 
     def paint_jpeg(self, img_data, x, y, width, height, options, callbacks):
-        img = self.jpeg_decoder.decompress_to_rgb("RGBX", img_data)
+        self.do_paint_jpeg("jpeg", img_data, x, y, width, height, options, callbacks)
+
+    def paint_jpega(self, img_data, x, y, width, height, options, callbacks):
+        self.do_paint_jpeg("jpega", img_data, x, y, width, height, options, callbacks)
+
+    def do_paint_jpeg(self, encoding, img_data, x, y, width, height, options, callbacks):
+        alpha_offset = options.intget("alpha-offset", 0)
+        log("do_paint_jpeg: nvjpeg_decoder=%s", self.nvjpeg_decoder)
+        if self.nvjpeg_decoder and not alpha_offset:
+            with self.assign_cuda_context(False):
+                img = self.nvjpeg_decoder.decompress_and_download("RGB", img_data)
+        else:
+            if encoding=="jpeg":
+                rgb_format = "RGBX"
+            elif encoding=="jpega":
+                rgb_format = "RGBA"
+            else:
+                raise Exception("invalid encoding %r" % encoding)
+            img = self.jpeg_decoder.decompress_to_rgb(rgb_format, img_data, alpha_offset)
+        rgb_format = img.get_pixel_format()
+        img_data = img.get_pixels()
+        rowstride = img.get_rowstride()
+        w = img.get_width()
+        h = img.get_height()
+        self.idle_add(self.do_paint_rgb, rgb_format, img_data,
+                  x, y, w, h, width, height, rowstride, options, callbacks)
+
+    def paint_avif(self, img_data, x, y, width, height, options, callbacks):
+        img = self.avif_decoder.decompress(img_data, options)
         rgb_format = img.get_pixel_format()
         img_data = img.get_pixels()
         rowstride = img.get_rowstride()
@@ -360,13 +502,19 @@ class WindowBackingBase:
         self.idle_add(self.do_paint_rgb, rgb_format, img_data,
                       x, y, w, h, width, height, rowstride, options, callbacks)
 
-
     def paint_image(self, coding, img_data, x, y, width, height, options, callbacks):
         # can be called from any thread
         rgb_format, img_data, iwidth, iheight, rowstride = self.pil_decoder.decompress(coding, img_data, options)
         self.idle_add(self.do_paint_rgb, rgb_format, img_data,
                       x, y, iwidth, iheight, width, height, rowstride, options, callbacks)
 
+    def paint_spng(self, img_data, x, y, width, height, options, callbacks):
+        rgba, rgb_format, iwidth, iheight = self.spng_decoder.decompress(img_data)
+        rowstride = iwidth*len(rgb_format)
+        self.idle_add(self.do_paint_rgb, rgb_format, rgba,
+                      x, y, iwidth, iheight, width, height, rowstride, options, callbacks)
+
+
     def paint_webp(self, img_data, x, y, width, height, options, callbacks):
         if not self.webp_decoder or WEBP_PILLOW:
             #if webp is enabled, then Pillow should be able to take care of it:
@@ -378,24 +526,25 @@ class WindowBackingBase:
             buffer_wrapper,
             iwidth, iheight, stride, has_alpha,
             rgb_format,
-            ) = self.webp_decoder.decompress(img_data, has_alpha, rgb_format, self.RGB_MODES)
+            ) = self.webp_decoder.decompress(img_data, has_alpha, rgb_format, self.get_rgb_formats())
         def free_buffer(*_args):
             buffer_wrapper.free()
         callbacks.append(free_buffer)
         data = buffer_wrapper.get_pixels()
         #if the backing can't handle this format,
         #ie: tray only supports RGBA
-        if rgb_format not in self.RGB_MODES:
+        if rgb_format not in self.get_rgb_formats():
+            # pylint: disable=import-outside-toplevel
             from xpra.codecs.rgb_transform import rgb_reformat
             from xpra.codecs.image_wrapper import ImageWrapper
             img = ImageWrapper(x, y, iwidth, iheight, data, rgb_format,
                                len(rgb_format)*8, stride, len(rgb_format), ImageWrapper.PACKED, True, None)
-            rgb_reformat(img, self.RGB_MODES, has_alpha and self._alpha_enabled)
+            rgb_reformat(img, self.get_rgb_formats(), has_alpha and self._alpha_enabled)
             rgb_format = img.get_pixel_format()
             data = img.get_pixels()
             stride = img.get_rowstride()
         #replace with the actual rgb format we get from the decoder:
-        options[b"rgb_format"] = rgb_format
+        options["rgb_format"] = rgb_format
         self.idle_add(self.do_paint_rgb, rgb_format, data,
                                  x, y, iwidth, iheight, width, height, stride, options, callbacks)
 
@@ -442,7 +591,7 @@ class WindowBackingBase:
                 paint_fn = self._do_paint_rgb32
             else:
                 raise Exception("invalid rgb format '%s'" % rgb_format)
-            options[b"rgb_format"] = rgb_format
+            options["rgb_format"] = rgb_format
             success = paint_fn(img_data, x, y, width, height, render_width, render_height, rowstride, options)
             fire_paint_callbacks(callbacks, success)
         except Exception as e:
@@ -454,16 +603,16 @@ class WindowBackingBase:
                 fire_paint_callbacks(callbacks, False, message)
 
     def _do_paint_rgb16(self, img_data, x, y, width, height, render_width, render_height, rowstride, options):
-        raise Exception("override me!")
+        raise NotImplementedError
 
     def _do_paint_rgb24(self, img_data, x, y, width, height, render_width, render_height, rowstride, options):
-        raise Exception("override me!")
+        raise NotImplementedError
 
     def _do_paint_rgb30(self, img_data, x, y, width, height, render_width, render_height, rowstride, options):
-        raise Exception("override me!")
+        raise NotImplementedError
 
     def _do_paint_rgb32(self, img_data, x, y, width, height, render_width, render_height, rowstride, options):
-        raise Exception("override me!")
+        raise NotImplementedError
 
 
     def eos(self):
@@ -493,14 +642,15 @@ class WindowBackingBase:
                 v = self.validate_csc_size(spec, src_width, src_height, dst_width, dst_height)
                 if v:
                     continue
+                options = {"speed" : speed}
                 try:
                     csc = spec.make_instance()
                     csc.init_context(src_width, src_height, src_format,
-                               dst_width, dst_height, dst_format, speed)
+                               dst_width, dst_height, dst_format, options)
                     return csc
                 except Exception as e:
                     videolog("make_csc%s",
-                        (src_width, src_height, src_format, dst_width, dst_height, dst_format_options, speed),
+                        (src_width, src_height, src_format, dst_width, dst_height, dst_format_options, options),
                         exc_info=True)
                     videolog.error("Error: failed to create csc instance %s", spec.codec_class)
                     videolog.error(" for %s to %s: %s", src_format, dst_format, e)
@@ -557,12 +707,7 @@ class WindowBackingBase:
                 fire_paint_callbacks(callbacks, -1, message)
                 return
             enc_width, enc_height = options.intpair("scaled_size", (width, height))
-            input_colorspace = options.strget("csc")
-            if not input_colorspace:
-                message = "csc mode is missing from the video options!"
-                log.error(message)
-                fire_paint_callbacks(callbacks, False, message)
-                return
+            input_colorspace = options.strget("csc", "YUV420P")
             #do we need a prep step for decoders that cannot handle the input_colorspace directly?
             decoder_colorspaces = decoder_module.get_input_colorspaces(coding)
             assert input_colorspace in decoder_colorspaces, "decoder %s does not support %s for %s" % (
@@ -618,7 +763,7 @@ class WindowBackingBase:
             self.close_decoder(True)
 
     def do_video_paint(self, img, x, y, enc_width, enc_height, width, height, options, callbacks):
-        target_rgb_formats = self.RGB_MODES
+        target_rgb_formats = self.get_rgb_formats()
         #as some video formats like vpx can forward transparency
         #also we could skip the csc step in some cases:
         pixel_format = img.get_pixel_format()
@@ -671,7 +816,7 @@ class WindowBackingBase:
             see _mmap_send() in server.py for details """
         assert self.mmap_enabled
         data = mmap_read(self.mmap, *img_data)
-        rgb_format = options.strget(b"rgb_format", b"RGB")
+        rgb_format = options.strget("rgb_format", "RGB")
         #Note: BGR(A) is only handled by gl_window_backing
         x, y = self.gravity_adjust(x, y, options)
         self.do_paint_rgb(rgb_format, data, x, y, width, height, width, height, rowstride, options, callbacks)
@@ -695,7 +840,7 @@ class WindowBackingBase:
                 self.idle_add(self.paint_mmap, img_data, x, y, width, height, rowstride, options, callbacks)
             elif coding in ("rgb24", "rgb32"):
                 #avoid confusion over how many bytes-per-pixel we may have:
-                rgb_format = options.strget(b"rgb_format")
+                rgb_format = options.strget("rgb_format")
                 if not rgb_format:
                     rgb_format = {
                         "rgb24" : "RGB",
@@ -710,8 +855,14 @@ class WindowBackingBase:
                                               img_data, x, y, width, height, options, callbacks)
             elif self.jpeg_decoder and coding=="jpeg":
                 self.paint_jpeg(img_data, x, y, width, height, options, callbacks)
+            elif self.jpeg_decoder and coding=="jpega":
+                self.paint_jpega(img_data, x, y, width, height, options, callbacks)
+            elif self.avif_decoder and coding=="avif":
+                self.paint_avif(img_data, x, y, width, height, options, callbacks)
             elif coding == "webp":
                 self.paint_webp(img_data, x, y, width, height, options, callbacks)
+            elif self.spng_decoder and coding=="png":
+                self.paint_spng(img_data, x, y, width, height, options, callbacks)
             elif coding in self._PIL_encodings:
                 self.paint_image(coding, img_data, x, y, width, height, options, callbacks)
             elif coding == "scroll":
