@@ -38,7 +38,7 @@ def parse_link(content):
 
 def read_link(path):
     try:
-        with open(path, 'rb') as stream:
+        with open(path, "rb") as stream:
             content = stream.read()
         return parse_link(content)
     except Exception as e:
@@ -121,6 +121,7 @@ def load_menu():
 
 
 def main():
+    # pylint: disable=import-outside-toplevel
     from xpra.platform import program_context
     with program_context("menu-helper", "Menu Helper"):
         menu = load_menu()
