@@ -7,9 +7,9 @@
 # Augments the win32_NotifyIcon "system tray" support class
 # with methods for integrating with win32_balloon and the popup menu
 
+from time import monotonic
 from ctypes import wintypes, byref
 
-from xpra.os_util import monotonic_time
 from xpra.platform.win32 import constants as win32con
 from xpra.platform.win32.gui import EnumDisplayMonitors, GetMonitorInfo
 from xpra.platform.win32.win32_NotifyIcon import win32NotifyIcon
@@ -29,19 +29,17 @@ class Win32Tray(TrayBase):
         self.calculate_offset()
         self.default_icon_extension = "ico"
         icon_filename = get_icon_filename(self.default_icon_filename, "ico")
-        self.tray_widget = win32NotifyIcon(self.app_id, self.tooltip, self.move_cb, self.click_cb, self.exit_cb, icon_filename)
+        self.tray_widget = win32NotifyIcon(self.app_id, self.tooltip,
+                                           self.move_cb, self.click_cb, self.exit_cb, icon_filename)
         el = get_win32_event_listener()
         if el:
             el.add_event_callback(win32con.WM_DISPLAYCHANGE, self.calculate_offset)
 
-    def ready(self):
-        pass
-
     def show(self):
-        pass
+        """ we can't hide or show the tray on MS Windows """
 
     def hide(self):
-        pass
+        """ we can't hide or show the tray on MS Windows """
 
 
     def get_size(self):
@@ -87,18 +85,23 @@ class Win32Tray(TrayBase):
     def set_icon_from_data(self, pixels, has_alpha, w, h, rowstride, options=None):
         if self.tray_widget:
             self.tray_widget.set_icon_from_data(pixels, has_alpha, w, h, rowstride, options)
-            self.icon_timestamp = monotonic_time()
+            self.icon_timestamp = monotonic()
 
     def do_set_icon_from_file(self, filename):
         if self.tray_widget:
             self.tray_widget.set_icon(filename)
-            self.icon_timestamp = monotonic_time()
+            self.icon_timestamp = monotonic()
 
     def set_blinking(self, on):
         if self.tray_widget:
             self.tray_widget.set_blinking(on)
 
     def get_geometry(self):
+        tw = self.tray_widget
+        if tw:
+            geom = tw.get_geometry()
+            if geom:
+                return geom
         return self.geometry_guess
 
 
