@@ -1,11 +1,12 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2022 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 #pylint: disable-msg=E1101
 
 from xpra.scripts.config import parse_bool_or_int
+from xpra.os_util import bytestostr, OSX
 from xpra.codecs.codec_constants import PREFERRED_ENCODING_ORDER
 from xpra.codecs.loader import get_codec, has_codec, codec_versions, load_codec
 from xpra.codecs.video_helper import getVideoHelper
@@ -59,8 +60,10 @@ class EncodingServer(StubServerMixin):
         if "jpeg" in ae:
             #try to load the fast jpeg encoders:
             load_codec("enc_jpeg")
+        if "avif" in ae:
+            load_codec("enc_avif")
         self.init_encodings()
-        self.init_thread_callbacks.append(self.reinit_encodings)
+        self.add_init_thread_callback(self.reinit_encodings)
 
     def reinit_encodings(self):
         self.init_encodings()
@@ -73,7 +76,7 @@ class EncodingServer(StubServerMixin):
 
     def threaded_setup(self):
         #load the slower codecs
-        if "jpeg" in self.allowed_encodings:
+        if "jpeg" in self.allowed_encodings and not OSX:
             load_codec("enc_nvjpeg")
         #load video codecs:
         getVideoHelper().init()
@@ -160,6 +163,10 @@ class EncodingServer(StubServerMixin):
                 add_encodings("webp")
                 if "webp" not in lossless:
                     lossless.append("webp")
+        for codec_name in ("avif", "enc_jpeg", "enc_nvjpeg"):
+            codec = get_codec(codec_name)
+            if codec:
+                add_encodings(*codec.get_encodings())
         #look for video encodings with lossless mode:
         for e in ve:
             for colorspace,especs in getVideoHelper().get_encoder_specs(e).items():
@@ -193,7 +200,7 @@ class EncodingServer(StubServerMixin):
 
 
     def _process_encoding(self, proto, packet):
-        encoding = packet[1].decode("latin1")
+        encoding = bytestostr(packet[1])
         ss = self.get_server_source(proto)
         if ss is None:
             return
@@ -214,38 +221,21 @@ class EncodingServer(StubServerMixin):
         ss.set_encoding(encoding, wids)
         self._refresh_windows(proto, wid_windows, {})
 
-    def _process_quality(self, proto, packet):
-        quality = packet[1]
-        log("Setting quality to %s", quality)
+    def _modify_sq(self, proto, packet):
+        """ modify speed or quality """
         ss = self.get_server_source(proto)
-        if ss:
-            ss.set_quality(quality)
-            self.call_idle_refresh_all_windows(proto)
-
-    def _process_min_quality(self, proto, packet):
-        min_quality = packet[1]
-        log("Setting min quality to %s", min_quality)
-        ss = self.get_server_source(proto)
-        if ss:
-            ss.set_min_quality(min_quality)
-            self.call_idle_refresh_all_windows(proto)
-
-    def _process_speed(self, proto, packet):
-        speed = packet[1]
-        log("Setting speed to ", speed)
-        ss = self.get_server_source(proto)
-        if ss:
-            ss.set_speed(speed)
-            self.call_idle_refresh_all_windows(proto)
-
-    def _process_min_speed(self, proto, packet):
-        min_speed = packet[1]
-        log("Setting min speed to ", min_speed)
-        ss = self.get_server_source(proto)
-        if ss:
-            ss.set_min_speed(min_speed)
-            self.call_idle_refresh_all_windows(proto)
-
+        if not ss:
+            return
+        ptype = bytestostr(packet[0])
+        assert ptype in (
+            "quality", "min-quality", "max-quality",
+            "speed", "min-speed", "max-speed",
+            )
+        value = int(packet[1])
+        log("Setting %s to %s", ptype, value)
+        fn = getattr(ss, "set_%s" % ptype.replace("-", "_"))
+        fn(value)
+        self.call_idle_refresh_all_windows(proto)
 
     def call_idle_refresh_all_windows(self, proto):
         #we can't assume that the window server mixin is loaded:
@@ -255,10 +245,8 @@ class EncodingServer(StubServerMixin):
 
 
     def init_packet_handlers(self):
-        self.add_packet_handlers({
-            "quality"       : self._process_quality,
-            "min-quality"   : self._process_min_quality,
-            "speed"         : self._process_speed,
-            "min-speed"     : self._process_min_speed,
-            "encoding"      : self._process_encoding,
-            })
+        for ptype in (
+            "quality", "min-quality", "max-quality",
+            "speed", "min-speed", "max-speed",
+            ):
+            self.add_packet_handler(ptype, self._modify_sq)
