@@ -4,8 +4,10 @@
 # later version. See the file COPYING for details.
 
 import sys
+import json
 import shlex
 import os.path
+from subprocess import Popen, PIPE
 
 from xpra.util import envbool
 from xpra.os_util import (
@@ -14,10 +16,11 @@ from xpra.os_util import (
     shellsub,
     get_util_logger,
     osexpand, umask_context,
-    close_all_fds,
     )
+from xpra.log import Logger
 from xpra.platform.dotxpra import norm_makepath
-from xpra.scripts.config import InitException
+from xpra.platform.paths import get_python_exec_command
+from xpra.scripts.config import InitException, FALSE_OPTIONS
 
 UINPUT_UUID_LEN = 12
 
@@ -27,85 +30,96 @@ def source_env(source=()) -> dict:
     log("source_env(%s)", source)
     env = {}
     for f in source:
-        if not f:
+        if not f or f.lower() in FALSE_OPTIONS:
             continue
-        e = env_from_sourcing(f)
-        log("source_env %s=%s", f, e)
-        env.update(e)
+        try:
+            e = env_from_sourcing(f)
+        except Exception as e:
+            log(f"env_from_sourcing({f})", exc_info=True)
+            log.error(f"Error sourcing {f!r}: {e}")
+        else:
+            log("source_env %s=%s", f, e)
+            env.update(e)
     log("source_env(%s)=%s", source, env)
     return env
 
 
+def decode_dict(out):
+    env = {}
+    for line in out.splitlines():
+        parts = line.split("=", 1)
+        if len(parts)==2:
+            env[parts[0]] = parts[1]
+    return env
+
+def decode_json(out):
+    return json.loads(out)
+
+
 # credit: https://stackoverflow.com/a/47080959/428751
 # returns a dictionary of the environment variables resulting from sourcing a file
 def env_from_sourcing(file_to_source_path, include_unexported_variables=False):
-    import json
-    import subprocess
-    from xpra.log import Logger
     log = Logger("exec")
     cmd = shlex.split(file_to_source_path)
-    filename = which(cmd[0])
-    if not filename:
-        log.error("Error: cannot find command '%s' to execute", cmd[0])
-        log.error(" for sourcing '%s'", file_to_source_path)
-        return {}
+    filename = cmd[0]
     if not os.path.isabs(filename):
-        filename = os.path.abspath(filename)
+        filename = which(filename)
+        if not filename:
+            log.error("Error: cannot find command '%s' to execute", cmd[0])
+            log.error(" for sourcing '%s'", file_to_source_path)
+            return {}
+        if not os.path.isabs(filename):
+            filename = os.path.abspath(filename)
     cmd[0] = filename
     #figure out if this is a script to source,
     #or if we're meant to execute it directly
     try:
-        with open(filename, 'rb') as f:
+        with open(filename, "rb") as f:
             first_line = f.readline()
     except OSError as e:
-        log.error("Error: failed to read from '%s'", filename)
-        log.error(" %s", e)
+        log.error(f"Error: failed to read from {filename!r}")
+        log.estr(e)
         first_line = b""
     else:
-        log("first line of '%s': %r", filename, first_line)
+        log(f"first line of {filename!r}: {first_line!r}")
     if first_line.startswith(b"\x7fELF") or b"\x00" in first_line:
-        def decode(out):
-            env = {}
-            for line in out.splitlines():
-                parts = line.split("=", 1)
-                if len(parts)==2:
-                    env[parts[0]] = parts[1]
-            log("decode(%r)=%s", out, env)
-            return env
+        decode = decode_dict
     else:
-        source = '%ssource %s' % ("set -a && " if include_unexported_variables else "", filename)
-        dump = 'python%i.%i -c "import os, json;print(json.dumps(dict(os.environ)))"' % (sys.version_info.major, sys.version_info.minor)
-        cmd = ['/bin/bash', '-c', '%s && %s' % (source, dump)]
-        def decode(out):
-            try:
-                env = json.loads(out)
-                log("json loads(%r)=%s", out, env)
-                return env
-            except json.decoder.JSONDecodeError:
-                log.error("Error decoding json output from sourcing script '%s'",
-                          file_to_source_path, exc_info=True)
-                return {}
+        source = '%s. %s' % ("set -a && " if include_unexported_variables else "", filename)
+        #ie: this is "python3.9 -c" on Posix
+        #(but our 'Python_exec_cmd.exe' wrapper on MS Windows):
+        python_cmd = " ".join(get_python_exec_command())
+        dump = '%s "import os, json;print(json.dumps(dict(os.environ)))"' % (python_cmd)
+        sh = which("bash") or "/bin/sh"
+        cmd = [sh, '-c', '%s 1>&2 && %s' % (source, dump)]
+        decode = decode_json
+    out = err = ""
     try:
         log("env_from_sourcing%s cmd=%s", (filename, include_unexported_variables), cmd)
-        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE)
-        out = proc.communicate()[0]
+        proc = Popen(cmd, stdout=PIPE, stderr=PIPE)
+        out, err = proc.communicate()
         if proc.returncode!=0:
-            log.error("Error %i running source script '%s'", proc.returncode, filename)
+            log.error(f"Error {proc.returncode} running source script {filename!r}")
     except OSError as e:
         log("env_from_sourcing%s", (filename, include_unexported_variables), exc_info=True)
-        log(" stdout=%r (%s)", out, type(out))
-        log.error("Error running source script '%s'", proc.returncode, file_to_source_path)
-        log.error(" %s", e)
+        log(f" stdout={out} ({type(out)})")
+        log(f" stderr={err} ({type(err)})")
+        log.error(f"Error {proc.returncode} running source script {file_to_source_path!r}")
+        log.error(f" {e}")
         return {}
-    log("stdout(%s)=%r", filename, out)
-    if not out:
-        return {}
-    try:
-        out = out.decode()
-    except UnicodeDecodeError:
-        log.error("Error decoding output from '%s'", filename, exc_info=True)
-        return {}
-    return decode(out)
+    log(f"stdout({filename})={out}")
+    log(f"stderr({filename})={err}")
+    def proc_str(b, fdname="stdout"):
+        try:
+            return (b or b"").decode()
+        except UnicodeDecodeError:
+            log.error(f"Error decoding {fdname} from {filename!r}", exc_info=True)
+        return ""
+    env = {}
+    env.update(decode(proc_str(out, "stdout")))
+    env.update(decode_dict(proc_str(err, "stderr")))
+    log("env_from_sourcing%s=%s", (file_to_source_path, include_unexported_variables), env)
+    return env
 
 
 def sh_quotemeta(s):
@@ -268,24 +282,33 @@ def select_log_file(log_dir, log_file, display_name):
         logpath = os.path.join(log_dir, "tmp_%d.log" % os.getpid())
     return logpath
 
-def redirect_std_to_log(logfd, *noclose_fds):
-    # save current stdout/stderr to be able to print info
-    # before exiting the non-deamon process
-    # and closing those file descriptors definitively
+# Redirects stdin from /dev/null, and stdout and stderr to the file with the
+# given file descriptor. Returns file objects pointing to the old stdout and
+# stderr, which can be used to write a message about the redirection.
+def redirect_std_to_log(logfd):
+    # preserve old stdio in new filehandles for use (and subsequent closing)
+    # by the caller
     old_fd_stdout = os.dup(1)
     old_fd_stderr = os.dup(2)
-    close_all_fds(exceptions=[logfd, old_fd_stdout, old_fd_stderr]+list(noclose_fds))
+    stdout = os.fdopen(old_fd_stdout, "w", 1)
+    stderr = os.fdopen(old_fd_stderr, "w", 1)
+
+    # close the old stdio file handles
+    os.close(0)
+    os.close(1)
+    os.close(2)
+
+    # replace stdin with /dev/null
     fd0 = os.open("/dev/null", os.O_RDONLY)
     if fd0 != 0:
         os.dup2(fd0, 0)
         os.close(fd0)
-    # reopen STDIO files
-    stdout = os.fdopen(old_fd_stdout, "w", 1)
-    stderr = os.fdopen(old_fd_stderr, "w", 1)
+
     # replace standard stdout/stderr by the log file
     os.dup2(logfd, 1)
     os.dup2(logfd, 2)
     os.close(logfd)
+
     # Make these line-buffered:
     sys.stdout = os.fdopen(1, "w", 1)
     sys.stderr = os.fdopen(2, "w", 1)
@@ -314,10 +337,11 @@ def write_pidfile(pidfile):
                 inode = os.fstat(f.fileno()).st_ino
             except OSError:
                 inode = 0
-        log.info("wrote pid %s to '%s'", pidstr, pidfile)
+        log.info(f"wrote pid {pidstr} to {pidfile!r}")
     except Exception as e:
-        log.error("Error: failed to write pid %i to pidfile '%s':", os.getpid(), pidfile)
-        log.error(" %s", e)
+        log(f"write_pidfile({pidfile})", exc_info=True)
+        log.info(f"Error: failed to write pid {pidstr} to {pidfile!r}")
+        log.error(f" {e}")
     return inode
 
 def rm_pidfile(pidfile, inode):
@@ -389,14 +413,15 @@ def has_uinput():
     except ImportError as e:
         log = get_util_logger()
         log("has_uinput()", exc_info=True)
-        log.info("no uinput module")
+        log.info("no uinput module (not usually needed)")
         return False
     try:
         uinput.fdopen()         #@UndefinedVariable
     except Exception as e:
         log = get_util_logger()
         log("has_uinput()", exc_info=True)
-        log.info("cannot use uinput for virtual devices:")
+        log.info("cannot use uinput for virtual devices,")
+        log.info(" this is usually a permission issue:")
         log.info(" %s", e)
         return False
     return True
@@ -408,7 +433,7 @@ def create_uinput_device(uuid, uid, events, name):
     #BUS_VIRTUAL = 0x06
     VENDOR = 0xffff
     PRODUCT = 0x1000
-    #our xpra_udev_product_version script will use the version attribute to set
+    #our 'udev_product_version' script will use the version attribute to set
     #the udev OWNER value
     VERSION = uid
     try:
