@@ -53,7 +53,6 @@ class AudioMixin(StubSourceMixin):
         self.sound_encoders = ()
         self.sound_receive = False
         self.sound_send = False
-        self.sound_bundle_metadata = False
         self.sound_fade_timer = None
         self.new_stream_timers = {}
 
@@ -77,7 +76,7 @@ class AudioMixin(StubSourceMixin):
 
     def stop_new_stream_notification(self, proc):
         r = proc.poll()
-        log.info("stop_new_stream_notification(%s) exit code=%s", proc, r)
+        log("stop_new_stream_notification(%s) exit code=%s", proc, r)
         if r is not None:
             #already ended
             return
@@ -88,15 +87,26 @@ class AudioMixin(StubSourceMixin):
 
 
     def parse_client_caps(self, c):
-        self.wants_sound = c.boolget("wants_sound", True)
-        self.pulseaudio_id = c.strget("sound.pulseaudio.id")
-        self.pulseaudio_cookie_hash = c.strget("sound.pulseaudio.cookie-hash")
-        self.pulseaudio_server = c.strget("sound.pulseaudio.server")
-        self.sound_decoders = c.strtupleget("sound.decoders", [])
-        self.sound_encoders = c.strtupleget("sound.encoders", [])
-        self.sound_receive = c.boolget("sound.receive")
-        self.sound_send = c.boolget("sound.send")
-        self.sound_bundle_metadata = c.boolget("sound.bundle-metadata")
+        self.wants_sound = c.boolget("wants_sound", True) or "sound" in c.strtupleget("wants")
+        audio = c.dictget("audio")
+        if audio:
+            audio = typedict(audio)
+            self.pulseaudio_id = audio.strget("pulseaudio.id")
+            self.pulseaudio_cookie_hash = audio.strget("pulseaudio.cookie-hash")
+            self.pulseaudio_server = audio.strget("pulseaudio.server")
+            self.sound_decoders = audio.strtupleget("decoders", [])
+            self.sound_encoders = audio.strtupleget("encoders", [])
+            self.sound_receive = audio.boolget("receive")
+            self.sound_send = audio.boolget("send")
+        else:
+            #pre v4.4:
+            self.pulseaudio_id = c.strget("sound.pulseaudio.id")
+            self.pulseaudio_cookie_hash = c.strget("sound.pulseaudio.cookie-hash")
+            self.pulseaudio_server = c.strget("sound.pulseaudio.server")
+            self.sound_decoders = c.strtupleget("sound.decoders", [])
+            self.sound_encoders = c.strtupleget("sound.encoders", [])
+            self.sound_receive = c.boolget("sound.receive")
+            self.sound_send = c.boolget("sound.send")
         log("pulseaudio id=%s, cookie-hash=%s, server=%s, sound decoders=%s, sound encoders=%s, receive=%s, send=%s",
                  self.pulseaudio_id, self.pulseaudio_cookie_hash, self.pulseaudio_server,
                  self.sound_decoders, self.sound_encoders, self.sound_receive, self.sound_send)
@@ -112,7 +122,9 @@ class AudioMixin(StubSourceMixin):
             "send"              : self.supports_speaker and len(self.speaker_codecs)>0,
             "receive"           : self.supports_microphone and len(self.microphone_codecs)>0,
             })
-        return flatten_dict({"sound" : sound_props})
+        caps = flatten_dict({"sound" : sound_props})
+        caps["audio"] = sound_props
+        return caps
 
 
     def audio_loop_check(self, mode="speaker") -> bool:
@@ -242,6 +254,7 @@ class AudioMixin(StubSourceMixin):
             self.send_eos(ss.codec, ss.sequence)
             self.sound_source_sequence += 1
             ss.cleanup()
+        self.call_update_av_sync_delay()
 
     def send_eos(self, codec, sequence=0):
         log("send_eos(%s, %s)", codec, sequence)
@@ -280,8 +293,10 @@ class AudioMixin(StubSourceMixin):
                             self.stop_new_stream_notification(proc)
                     timer = self.timeout_add(NEW_STREAM_SOUND_STOP*1000, stop_new_stream_notification)
                     self.new_stream_timers[proc] = timer
-            except Exception:
-                pass
+            except Exception as e:
+                log("new_stream(%s, %s) error playing new stream sound", sound_source, codec, exc_info=True)
+                log.error("Error playing new-stream bell sound:")
+                log.error(" %s", e)
         log("new_stream(%s, %s)", sound_source, codec)
         if self.sound_source!=sound_source:
             log("dropping new-stream signal (current source=%s, signal source=%s)", self.sound_source, sound_source)
@@ -295,13 +310,19 @@ class AudioMixin(StubSourceMixin):
                    "codec"              : codec,
                    "sequence"           : sound_source.sequence,
                    })
+        self.call_update_av_sync_delay()
+        #run it again after 10 seconds,
+        #by that point the source info will actually be populated:
+        from gi.repository import GLib
+        GLib.timeout_add(10*1000, self.call_update_av_sync_delay)
+
+    def call_update_av_sync_delay(self):
+        #loose coupling with avsync mixin:
         update_av_sync = getattr(self, "update_av_sync_delay_total", None)
-        if update_av_sync:
+        log("call_update_av_sync_delay update_av_sync=%s", update_av_sync)
+        if callable(update_av_sync):
             update_av_sync()  #pylint: disable=not-callable
-            #run it again after 10 seconds,
-            #by that point the source info will actually be populated:
-            from gi.repository import GLib
-            GLib.timeout_add(10*1000, update_av_sync)
+
 
     def new_sound_buffer(self, sound_source, data, metadata, packet_metadata=None):
         log("new_sound_buffer(%s, %s, %s, %s) info=%s",
@@ -314,23 +335,14 @@ class AudioMixin(StubSourceMixin):
                 sound_source.sequence, self.sound_source_sequence)
             return
         if packet_metadata:
-            if not self.sound_bundle_metadata:
-                #client does not support bundling, send packet metadata as individual packets before the main packet:
-                for x in packet_metadata:
-                    self.send_sound_data(sound_source, x, {})
-                packet_metadata = ()
-            else:
-                #the packet metadata is compressed already:
-                packet_metadata = Compressed("packet metadata", packet_metadata, can_inline=True)
+            #the packet metadata is compressed already:
+            packet_metadata = Compressed("packet metadata", packet_metadata, can_inline=True)
         #don't drop the first 10 buffers
         can_drop_packet = (sound_source.info or {}).get("buffer_count", 0)>10
         self.send_sound_data(sound_source, data, metadata, packet_metadata, can_drop_packet)
 
     def send_sound_data(self, sound_source, data, metadata, packet_metadata=None, can_drop_packet=False):
-        packet_data = [sound_source.codec, Compressed(sound_source.codec, data), metadata]
-        if packet_metadata:
-            assert self.sound_bundle_metadata
-            packet_data.append(packet_metadata)
+        packet_data = [sound_source.codec, Compressed(sound_source.codec, data), metadata, packet_metadata or ()]
         sequence = sound_source.sequence
         if sequence>=0:
             metadata["sequence"] = sequence
@@ -372,7 +384,8 @@ class AudioMixin(StubSourceMixin):
                 log.warn(msg)
                 return msg
             if sequence!=self.sound_source_sequence:
-                log.warn("sound sequence mismatch: %i vs %i", sequence, self.sound_source_sequence)
+                log.warn("Warning: sound sequence mismatch: %i vs %i",
+                         sequence, self.sound_source_sequence)
                 return "not stopped"
             log("stop: sequence number matches")
         self.stop_sending_sound()
@@ -459,18 +472,19 @@ class AudioMixin(StubSourceMixin):
         if not self.sound_sink:
             if not self.audio_loop_check("microphone"):
                 #make a fake object so we don't fire the audio loop check warning repeatedly
-                from xpra.util import AdHocStruct
-                self.sound_sink = AdHocStruct()
-                self.sound_sink.codec = codec
-                def noop(*_args):
-                    pass
-                self.sound_sink.add_data = noop
-                self.sound_sink.cleanup = noop
+                class FakeSink:
+                    def __init__(self, codec):
+                        self.codec = codec
+                    def add_data(self, *args):
+                        log("FakeSink.add_data%s ignored", args)
+                    def cleanup(self, *args):
+                        log("FakeSink.cleanup%s ignored", args)
+                self.sound_sink = FakeSink(codec)
                 return
             try:
                 def sound_sink_error(*args):
                     log("sound_sink_error%s", args)
-                    log.warn("stopping sound input because of error")
+                    log.warn("Warning: stopping sound input because of an error")
                     self.stop_receiving_sound()
                 from xpra.sound.wrapper import start_receiving_sound
                 ss = start_receiving_sound(codec)
@@ -482,13 +496,8 @@ class AudioMixin(StubSourceMixin):
                 ss.start()
                 log("sound_data(..) sound sink started")
             except Exception:
-                log.error("failed to setup sound", exc_info=True)
+                log.error("Error: failed to start receiving %r", codec, exc_info=True)
                 return
-        if packet_metadata:
-            if not self.sound_properties.boolget("bundle-metadata"):
-                for x in packet_metadata:
-                    self.sound_sink.add_data(x)
-                packet_metadata = ()
         self.sound_sink.add_data(data, metadata, packet_metadata)
 
 
