@@ -366,10 +366,11 @@ class BaseWindowModel(CoreX11WindowModel):
         group_leader = None
         if "window_group" in wm_hints:
             xid = wm_hints.get("window_group")
-            try:
-                group_leader = xid, get_pywindow(xid)
-            except Exception:
-                group_leader = xid, None
+            if xid:
+                try:
+                    group_leader = get_pywindow(xid)
+                except Exception:
+                    log.error("Error locating group leader window %#x", xid)
         self._updateprop("group-leader", group_leader)
         self._updateprop("attention-requested", wm_hints.get("urgency", False))
         _input = wm_hints.get("input")
@@ -654,12 +655,11 @@ class BaseWindowModel(CoreX11WindowModel):
             log("_NET_WM_FULLSCREEN_MONITORS: %s", event)
             #TODO: we should validate the indexes instead of copying them blindly!
             #TODO: keep track of source indication so we can forward that to the client
-            m1, m2, m3, m4 = event.data[0], event.data[1], event.data[2], event.data[3]
             N = 16      #FIXME: arbitrary limit
-            if m1<0 or m1>=N or m2<0 or m2>=N or m3<0 or m3>=N or m4<0 or m4>=N:
-                log.warn("invalid list of _NET_WM_FULLSCREEN_MONITORS - ignored")
+            monitors = list(event.data[:4])
+            if not all(0 <= x < N for x in monitors):
+                log.warn("Warning: invalid list of _NET_WM_FULLSCREEN_MONITORS:%s - ignored", event.data)
                 return False
-            monitors = [m1, m2, m3, m4]
             log("_NET_WM_FULLSCREEN_MONITORS: monitors=%s", monitors)
             prop_set(self.client_window, "_NET_WM_FULLSCREEN_MONITORS", ["u32"], monitors)
             return True
