@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2022 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -8,14 +8,16 @@
 #  http://lists.partiwm.org/pipermail/parti-discuss/2008-September/000041.html
 #  http://lists.partiwm.org/pipermail/parti-discuss/2008-September/000042.html
 # (also do not import anything that imports gtk)
+import glob
 import shlex
 import signal
 from time import monotonic
 from subprocess import Popen, PIPE, call
 import os.path
 
-from xpra.scripts.config import InitException, get_Xdummy_confdir
-from xpra.util import envbool, envint
+from xpra.common import RESOLUTION_ALIASES, DEFAULT_REFRESH_RATE, get_refresh_rate_for_value
+from xpra.scripts.config import InitException, get_Xdummy_confdir, FALSE_OPTIONS
+from xpra.util import envbool, envint, csv
 from xpra.os_util import (
     shellsub,
     setuidgid, getuid, getgid,
@@ -28,30 +30,53 @@ from xpra.log import Logger
 
 
 VFB_WAIT = envint("XPRA_VFB_WAIT", 3)
+XVFB_EXTRA_ARGS = os.environ.get("XPRA_XVFB_EXTRA_ARGS", "")
 
-RESOLUTION_ALIASES = {
-    "QVGA"  : (320, 240),
-    "VGA"   : (640, 480),
-    "SVGA"  : (800, 600),
-    "XGA"   : (1024, 768),
-    "1080P" : (1920, 1080),
-    "FHD"   : (1920, 1080),
-    "4K"    : (3840, 2160),
-    }
 
-def parse_resolution(s):
-    v = RESOLUTION_ALIASES.get(s.upper())
-    if v:
-        return v
-    res = tuple(int(x) for x in s.replace(",", "x").split("x", 1))
-    assert len(res)==2, "invalid resolution string '%s'" % s
-    return res
-def parse_env_resolution(envkey="XPRA_DEFAULT_VFB_RESOLUTION", default_res="8192x4096"):
-    s = os.environ.get(envkey, default_res)
-    return parse_resolution(s)
+def parse_resolution(s, default_refresh_rate=DEFAULT_REFRESH_RATE//1000):
+    if not s:
+        return None
+    s = s.upper()       #ie: 4K60
+    res_part = s
+    hz = get_refresh_rate_for_value(str(default_refresh_rate), DEFAULT_REFRESH_RATE)//1000
+    for sep in ("@", "K", "P"):
+        pos = s.find(sep)
+        if 0<pos<len(s)-1:
+            res_part, hz = s.split(sep, 1)
+            if sep!="@":
+                res_part += sep
+            break
+    res = RESOLUTION_ALIASES.get(res_part)
+    if not res:
+        try:
+            res = tuple(int(x) for x in res_part.replace(",", "x").split("X", 1))
+        except ValueError:
+            raise ValueError(f"failed to parse resolution {res_part!r}") from None
+        if len(res)!=2:
+            raise ValueError("invalid resolution string {s!r}")
+    res = list(res)
+    res.append(int(hz))
+    return tuple(res)
+def parse_resolutions(s, default_refresh_rate=DEFAULT_REFRESH_RATE//1000):
+    if not s or s.lower() in FALSE_OPTIONS:
+        return None
+    if s.lower() in ("none", "default"):
+        return ()
+    return tuple(parse_resolution(v, default_refresh_rate) for v in s.split(","))
+def parse_env_resolutions(envkey="XPRA_DEFAULT_VFB_RESOLUTIONS",
+                          single_envkey="XPRA_DEFAULT_VFB_RESOLUTION",
+                          default_res="8192x4096",
+                          default_refresh_rate=DEFAULT_REFRESH_RATE//1000):
+    s = os.environ.get(envkey)
+    if s:
+        return parse_resolutions(s, default_refresh_rate)
+    return (parse_resolution(os.environ.get(single_envkey, default_res), default_refresh_rate), )
 
-DEFAULT_VFB_RESOLUTION = parse_env_resolution()
-DEFAULT_DESKTOP_VFB_RESOLUTION = parse_env_resolution("XPRA_DEFAULT_DESKTOP_VFB_RESOLUTION", "1280x1024")
+def get_desktop_vfb_resolutions(default_refresh_rate=DEFAULT_REFRESH_RATE//1000):
+    return parse_env_resolutions("XPRA_DEFAULT_DESKTOP_VFB_RESOLUTIONS",
+                                 "XPRA_DEFAULT_DESKTOP_VFB_RESOLUTION",
+                                 "1280x1024",
+                                 default_refresh_rate=default_refresh_rate)
 PRIVATE_XAUTH = envbool("XPRA_PRIVATE_XAUTH", False)
 XAUTH_PER_DISPLAY = envbool("XPRA_XAUTH_PER_DISPLAY", True)
 
@@ -101,20 +126,20 @@ def create_xorg_device_configs(xorg_conf_dir, device_uuid, uid, gid):
 #create individual device files:
 def save_input_conf(xorg_conf_dir, i, dev_type, device_uuid, uid, gid):
     upper_dev_type = dev_type[:1].upper()+dev_type[1:]   #ie: Pointer
-    product_name = "Xpra Virtual %s %s" % (upper_dev_type, bytestostr(device_uuid))
-    identifier = "xpra-virtual-%s" % dev_type
-    conf_file = os.path.join(xorg_conf_dir, "%02i-%s.conf" % (i, dev_type))
-    with open(conf_file, "wb") as f:
-        f.write(strtobytes("""Section "InputClass"
-Identifier "%s"
-MatchProduct "%s"
+    product_name = f"Xpra Virtual {upper_dev_type} {bytestostr(device_uuid)}"
+    identifier = f"xpra-virtual-{dev_type}"
+    conf_file = os.path.join(xorg_conf_dir, f"{i:02}-{dev_type}.conf")
+    with open(conf_file, "w", encoding="utf8") as f:
+        f.write(f"""Section "InputClass"
+Identifier "{identifier}"
+MatchProduct "{product_name}"
 MatchUSBID "ffff:ffff"
-MatchIs%s "True"
+MatchIs{upper_dev_type} "True"
 Driver "libinput"
 Option "AccelProfile" "flat"
 Option "Ignore" "False"
 EndSection
-""" % (identifier, product_name, upper_dev_type)))
+""")
         os.fchown(f.fileno(), uid, gid)
     #Option "AccelerationProfile" "-1"
     #Option "AccelerationScheme" "none"
@@ -131,11 +156,12 @@ def get_xauthority_path(display_name, username, uid, gid):
         filename = pathexpand(filename)
         if os.path.exists(filename):
             return filename
+    # pylint: disable=import-outside-toplevel
     from xpra.platform.xposix.paths import _get_xpra_runtime_dir
     if PRIVATE_XAUTH:
         d = _get_xpra_runtime_dir()
         if XAUTH_PER_DISPLAY:
-            filename = "Xauthority-%s" % display_name.lstrip(":")
+            filename = "Xauthority-" + display_name.lstrip(":")
         else:
             filename = "Xauthority"
     else:
@@ -145,15 +171,19 @@ def get_xauthority_path(display_name, username, uid, gid):
 
 def start_Xvfb(xvfb_str, vfb_geom, pixel_depth, display_name, cwd, uid, gid, username, uinput_uuid=None):
     if not POSIX:
-        raise InitException("starting an Xvfb is not supported on %s" % os.name)
+        raise InitException(f"starting an Xvfb is not supported on {os.name}")
     if OSX:
         raise InitException("starting an Xvfb is not supported on MacOS")
     if not xvfb_str:
         raise InitException("the 'xvfb' command is not defined")
 
     log = get_vfb_logger()
-    log("start_Xvfb%s", (xvfb_str, vfb_geom, pixel_depth, display_name, cwd, uid, gid, username, uinput_uuid))
+    log("start_Xvfb%s XVFB_EXTRA_ARGS=%s",
+        (xvfb_str, vfb_geom, pixel_depth, display_name, cwd, uid, gid, username, uinput_uuid),
+        XVFB_EXTRA_ARGS)
     use_display_fd = display_name[0]=='S'
+    if XVFB_EXTRA_ARGS:
+        xvfb_str += " "+XVFB_EXTRA_ARGS
 
     subs = {}
     def pathexpand(s):
@@ -183,7 +213,7 @@ def start_Xvfb(xvfb_str, vfb_geom, pixel_depth, display_name, cwd, uid, gid, use
             no_arg = xvfb_cmd.index("0")
         except ValueError:
             no_arg = -1
-        geom_str = "%sx%s" % ("x".join(str(x) for x in vfb_geom), pixel_depth)
+        geom_str = "%sx%s" % ("x".join(str(x) for x in vfb_geom[:2]), pixel_depth)
         if no_arg>0 and xvfb_cmd[no_arg-1]=="-screen" and len(xvfb_cmd)>(no_arg+1):
             #found an existing "-screen" argument for this screen number,
             #replace it:
@@ -215,14 +245,14 @@ def start_Xvfb(xvfb_str, vfb_geom, pixel_depth, display_name, cwd, uid, gid, use
                     except OSError:
                         log("lchown(%s, %i, %i)", xorg_log_dir, uid, gid, exc_info=True)
             except OSError as e:
-                raise InitException("failed to create the Xorg log directory '%s': %s" % (xorg_log_dir, e)) from None
+                raise InitException(f"failed to create the Xorg log directory {xorg_log_dir!r}: {e}") from None
 
     if uinput_uuid:
         #use uinput:
         #identify -config xorg.conf argument and replace it with the uinput one:
         try:
             config_argindex = xvfb_cmd.index("-config")
-        except ValueError as e:
+        except ValueError:
             log.warn("Warning: cannot use uinput")
             log.warn(" '-config' argument not found in the xvfb command")
         else:
@@ -246,35 +276,37 @@ def start_Xvfb(xvfb_str, vfb_geom, pixel_depth, display_name, cwd, uid, gid, use
     try:
         if use_display_fd:
             def displayfd_err(msg):
-                raise InitException("%s: %s" % (xvfb_executable, msg))
+                raise InitException(f"{xvfb_executable}: {msg}")
             r_pipe, w_pipe = os.pipe()
             try:
                 os.set_inheritable(w_pipe, True)        #@UndefinedVariable
                 xvfb_cmd += ["-displayfd", str(w_pipe)]
-                xvfb_cmd[0] = "%s-for-Xpra-%s" % (xvfb_executable, display_name.lstrip(":"))
+                xvfb_cmd[0] = f"{xvfb_executable}-for-Xpra-" + display_name.lstrip(":")
                 def preexec():
                     os.setpgrp()
                     if getuid()==0 and uid:
                         setuidgid(uid, gid)
                 try:
+                    # pylint: disable=consider-using-with
+                    # pylint: disable=subprocess-popen-preexec-fn
                     xvfb = Popen(xvfb_cmd, executable=xvfb_executable,
                                  preexec_fn=preexec, cwd=cwd, pass_fds=(w_pipe,))
                 except OSError as e:
                     log("Popen%s", (xvfb_cmd, xvfb_executable, cwd), exc_info=True)
-                    raise InitException("failed to execute xvfb command %s: %s" % (xvfb_cmd, e)) from None
+                    raise InitException(f"failed to execute xvfb command {xvfb_cmd}: {e}") from None
                 assert xvfb.poll() is None, "xvfb command failed"
                 # Read the display number from the pipe we gave to Xvfb
                 try:
                     buf = read_displayfd(r_pipe)
                 except Exception as e:
                     log("read_displayfd(%s)", r_pipe, exc_info=True)
-                    displayfd_err("failed to read displayfd pipe %s: %s" % (r_pipe, e))
+                    displayfd_err(f"failed to read displayfd pipe {r_pipe}: {e}")
             finally:
                 osclose(r_pipe)
                 osclose(w_pipe)
             n = parse_displayfd(buf, displayfd_err)
-            new_display_name = ":%s" % n
-            log("Using display number provided by %s: %s", xvfb_executable, new_display_name)
+            new_display_name = f":{n}"
+            log(f"Using display number provided by {xvfb_executable}: {new_display_name}")
             if tmp_xorg_log_file:
                 #ie: ${HOME}/.xpra/Xorg.${DISPLAY}.log -> /home/antoine/.xpra/Xorg.S14700.log
                 f0 = shellsub(tmp_xorg_log_file, subs)
@@ -286,12 +318,12 @@ def start_Xvfb(xvfb_str, vfb_geom, pixel_depth, display_name, cwd, uid, gid, use
                         os.rename(f0, f1)
                     except Exception as e:
                         log.warn("Warning: failed to rename Xorg log file,")
-                        log.warn(" from '%s' to '%s'" % (f0, f1))
-                        log.warn(" %s" % e)
+                        log.warn(f" from {f0!r} to {f1!r}")
+                        log.warn(" {e}")
             display_name = new_display_name
         else:
             # use display specified
-            xvfb_cmd[0] = "%s-for-Xpra-%s" % (xvfb_executable, display_name)
+            xvfb_cmd[0] = f"{xvfb_executable}-for-Xpra-"+display_name.lstrip(":")
             xvfb_cmd.append(display_name)
             def preexec():
                 if getuid()==0 and (uid!=0 or gid!=0):
@@ -299,6 +331,8 @@ def start_Xvfb(xvfb_str, vfb_geom, pixel_depth, display_name, cwd, uid, gid, use
                 else:
                     os.setsid()
             log("xvfb_cmd=%s", xvfb_cmd)
+            # pylint: disable=consider-using-with
+            # pylint: disable=subprocess-popen-preexec-fn
             xvfb = Popen(xvfb_cmd, executable=xvfb_executable,
                          stdin=PIPE, preexec_fn=preexec)
     except Exception as e:
@@ -324,10 +358,11 @@ def kill_xvfb(xvfb_pid):
         os.unlink(xauthority)
 
 
-def set_initial_resolution(res=DEFAULT_VFB_RESOLUTION):
+def set_initial_resolution(resolutions):
     try:
         log = get_vfb_logger()
-        log("set_initial_resolution(%s)", res)
+        log("set_initial_resolution(%s)", resolutions)
+        #pylint: disable=import-outside-toplevel
         from xpra.x11.bindings.randr_bindings import RandRBindings      #@UnresolvedImport
         #try to set a reasonable display size:
         randr = RandRBindings()
@@ -335,21 +370,42 @@ def set_initial_resolution(res=DEFAULT_VFB_RESOLUTION):
             log.warn("Warning: no RandR support,")
             log.warn(" default virtual display size unchanged")
             return
+        if randr.is_dummy16():
+            monitors = {}
+            x, y = 0, 0
+            for i, res in enumerate(resolutions):
+                assert len(res)==3
+                if not (isinstance(v, int) for v in res):
+                    raise ValueError(f"resolution values must be ints, found: {res} ({csv(type(v) for v in res)})")
+                w, h, hz = res
+                monitors[i] = {
+                    "name"      : f"VFB-{i}",
+                    "primary"   : i==0,
+                    "geometry"  : (x, y, w, h),
+                    "refresh-rate" : hz*1000,
+                    "automatic" : True,
+                    }
+                x += w
+                #arranging vertically:
+                #y += h
+            randr.set_crtc_config(monitors)
+            return
+        res = resolutions[0][:2]
         sizes = randr.get_xrr_screen_sizes()
         size = randr.get_screen_size()
-        log("RandR available, current size=%s, sizes available=%s", size, sizes)
+        log(f"RandR available, current size={size}, sizes available={sizes}")
         if res not in sizes:
-            log.warn("Warning: cannot set resolution to %s", res)
+            log.warn(f"Warning: cannot set resolution to {res}")
             log.warn(" (this resolution is not available)")
         elif res==size:
-            log("initial resolution already set: %s", res)
+            log(f"initial resolution already set: {res}")
         else:
-            log("RandR setting new screen size to %s", res)
+            log(f"RandR setting new screen size to {res}")
             randr.set_screen_size(*res)
     except Exception as e:
-        log("set_initial_resolution(%s)", res, exc_info=True)
+        log(f"set_initial_resolution({resolutions})", exc_info=True)
         log.error("Error: failed to set the default screen size:")
-        log.error(" %s", e)
+        log.estr(e)
 
 
 def xauth_add(filename, display_name, xauth_data, uid, gid):
@@ -365,23 +421,23 @@ def xauth_add(filename, display_name, xauth_data, uid, gid):
         log("xauth command: %s", xauth_cmd)
         code = call(xauth_cmd, preexec_fn=preexec)
         end = monotonic()
-        if code!=0 and (end-start>=10):
-            log.warn("Warning: xauth command took %i seconds and failed" % (end-start))
+        elapsed = round(end-start)
+        if code!=0 and elapsed>=10:
+            log.warn(f"Warning: xauth command took {elapsed} seconds and failed")
             #took more than 10 seconds to fail, check for stale locks:
-            import glob
-            if glob.glob("%s-*" % filename):
+            if glob.glob(f"{filename}-*"):
                 log.warn("Warning: trying to clean some stale xauth locks")
                 xauth_cmd = ["xauth", "-b"]+xauth_args
-                log("xauth command: %s", xauth_cmd)
+                log(f"xauth command: {xauth_cmd}")
                 code = call(xauth_cmd, preexec_fn=preexec)
         if code!=0:
-            raise OSError("non-zero exit code: %s" % code)
+            raise OSError(f"non-zero exit code: {code}")
     except OSError as e:
         #trying to continue anyway!
         log = get_vfb_logger()
-        log.error("Error adding xauth entry for %s" % display_name)
+        log.error(f"Error adding xauth entry for {display_name}")
         log.error(" using command \"%s\":" % (" ".join(xauth_cmd)))
-        log.error(" %s" % (e,))
+        log.estr(e)
 
 def check_xvfb_process(xvfb=None, cmd="Xvfb", timeout=0, command=None):
     if xvfb is None:
@@ -410,8 +466,8 @@ def verify_display_ready(xvfb, display_name, shadowing_check=True, log_errors=Tr
         log = get_vfb_logger()
         log("verify_display_ready%s", (xvfb, display_name, shadowing_check), exc_info=True)
         if log_errors:
-            log.error("Error: failed to connect to display %s" % display_name)
-            log.error(" %s", e)
+            log.error(f"Error: failed to connect to display {display_name!r}")
+            log.estr(e)
         return False
     if shadowing_check and not check_xvfb_process(xvfb):
         #if we're here, there is an X11 server, but it isn't the one we started!
@@ -419,16 +475,17 @@ def verify_display_ready(xvfb, display_name, shadowing_check=True, log_errors=Tr
         log("verify_display_ready%s display exists, but the vfb process has terminated",
                 (xvfb, display_name, shadowing_check, log_errors))
         if log_errors:
-            log.error("There is an X11 server already running on display %s:" % display_name)
+            log.error(f"There is an X11 server already running on display {display_name}:")
             log.error("You may want to use:")
-            log.error("  'xpra upgrade %s' if an instance of xpra is still connected to it" % display_name)
-            log.error("  'xpra --use-display start %s' to connect xpra to an existing X11 server only" % display_name)
+            log.error(f"  'xpra upgrade {display_name}' if an instance of xpra is still connected to it")
+            log.error(f"  'xpra --use-display start {display_name}' to connect xpra to an existing X11 server only")
             log.error("")
         return False
     return True
 
 
 def main():
+    # pylint: disable=import-outside-toplevel
     import sys
     display = None
     if len(sys.argv)>1:
